@@ -1,5 +1,5 @@
 #!/bin/sh
 if [ -z "${REDECLIPSE_PATH+isset}" ]; then REDECLIPSE_PATH="$(cd "$(dirname "$0")" && pwd)"; fi
 REDECLIPSE_BINARY="redeclipse_server"
-REDECLIPSE_CALLED="true"
-. "${REDECLIPSE_PATH}/redeclipse.sh"
+export REDECLIPSE_BINARY
+REDECLIPSE_CALLED="true" . "${REDECLIPSE_PATH}/redeclipse.sh" $@
