@@ -118,7 +118,7 @@ struct BlendMapCache
 {
     BlendMapRoot node;
     int scale;
-    ivec origin;
+    ivec2 origin;
 };
 
 BlendMapCache *newblendmapcache() { return new BlendMapCache; }
@@ -131,7 +131,7 @@ bool setblendmaporigin(BlendMapCache *cache, const ivec &o, int size)
     {
         cache->node = blendmap;
         cache->scale = worldscale-BM_SCALE;
-        cache->origin = ivec(0, 0, 0);
+        cache->origin = ivec2(0, 0);
         return cache->node.solid!=&bmsolids[0xFF];
     }
 
@@ -150,7 +150,7 @@ bool setblendmaporigin(BlendMapCache *cache, const ivec &o, int size)
         {
             cache->node = BlendMapRoot(bm->type[n], bm->children[n]);
             cache->scale = bmscale;
-            cache->origin = ivec(x1&(~0U<<bmscale), y1&(~0U<<bmscale), 0);
+            cache->origin = ivec2(x1&(~0U<<bmscale), y1&(~0U<<bmscale));
             return cache->node.solid!=&bmsolids[0xFF];
         }
         bm = bm->children[n].branch;
@@ -159,7 +159,7 @@ bool setblendmaporigin(BlendMapCache *cache, const ivec &o, int size)
     cache->node.type = BM_BRANCH;
     cache->node.branch = bm;
     cache->scale = bmscale;
-    cache->origin = ivec(x1&(~0U<<bmscale), y1&(~0U<<bmscale), 0);
+    cache->origin = ivec2(x1&(~0U<<bmscale), y1&(~0U<<bmscale));
     return true;
 }
 
@@ -484,17 +484,13 @@ struct BlendBrush
     void gentex()
     {
         if(!tex) glGenTextures(1, &tex);
-        uchar *buf = new uchar[2*w*h];
+        uchar *buf = new uchar[w*h];
         uchar *dst = buf, *src = data;
         loopi(h)
         {
-            loopj(w)
-            {
-                *dst++ = 255 - *src;
-                *dst++ = 255 - *src++;
-            }
+            loopj(w) *dst++ = 255 - *src++;
         }
-        createtexture(tex, w, h, buf, 3, 1, GL_LUMINANCE_ALPHA);
+        createtexture(tex, w, h, buf, 3, 1, hasTRG ? GL_R8 : GL_LUMINANCE8);
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
         GLfloat border[4] = { 0, 0, 0, 0 };
