@@ -33,7 +33,7 @@ bool glaring = false;
 
 void drawglaretex()
 {
-    if(!glare || renderpath==R_FIXEDFUNCTION) return;
+    if(!glare) return;
 
     glaretex.render(1<<glaresize, 1<<glaresize, blurglare, blurglaresigma/100.0f);
 }
@@ -42,24 +42,18 @@ FVAR(IDF_PERSIST, glarescale, 0, 1, 8);
 
 void addglare()
 {
-    extern int viewtype;
-    if(!glare || viewtype || renderpath==R_FIXEDFUNCTION) return;
+    if(!glare) return;
 
     glEnable(GL_BLEND);
     glBlendFunc(GL_ONE, GL_ONE);
 
-    SETSHADER(glare);
+    SETSHADER(screenrect);
 
     glBindTexture(GL_TEXTURE_2D, glaretex.rendertex);
 
-    setlocalparamf("glarescale", SHPARAM_PIXEL, 0, glarescale, glarescale, glarescale);
+    gle::colorf(glarescale, glarescale, glarescale);
 
-    glBegin(GL_TRIANGLE_STRIP);
-    glTexCoord2f(0, 0); glVertex3f(-1, -1, 0);
-    glTexCoord2f(1, 0); glVertex3f( 1, -1, 0);
-    glTexCoord2f(0, 1); glVertex3f(-1,  1, 0);
-    glTexCoord2f(1, 1); glVertex3f( 1,  1, 0);
-    glEnd();
+    screenquad(1, 1);
 
     glDisable(GL_BLEND);
 }
