@@ -199,20 +199,17 @@ struct iqm : skelmodel, skelloader<iqm>
                     mpos += 3;
                     if(mtc)
                     {
-                        v.u = mtc[0];
-                        v.v = mtc[1];
+                        v.tc = vec2(mtc[0], mtc[1]);
                         mtc += 2;
                     }
-                    else v.u = v.v = 0;
+                    else v.tc = vec2(0, 0);
                     if(mnorm)
                     {
                         v.norm = vec(mnorm[0], -mnorm[1], mnorm[2]);
                         mnorm += 3;
                         if(mtan)
                         {
-                            bumpvert &bv = m->bumpverts[j];
-                            bv.tangent = vec(mtan[0], -mtan[1], mtan[2]);
-                            bv.bitangent = mtan[3];
+                            m->calctangent(m->bumpverts[j], v.norm, vec(mtan[0], -mtan[1], mtan[2]), mtan[3]);
                             mtan += 4;
                         }
                     }
@@ -380,16 +377,13 @@ struct iqm : skelmodel, skelloader<iqm>
 
     bool loaddefaultparts()
     {
-        skelpart &mdl = *new skelpart;
-        parts.add(&mdl);
-        mdl.model = this;
-        mdl.index = 0;
+        skelpart &mdl = addpart();
         mdl.pitchscale = mdl.pitchoffset = mdl.pitchmin = mdl.pitchmax = 0;
         adjustments.setsize(0);
-        const char *fname = loadname + strlen(loadname);
-        do --fname; while(fname >= loadname && *fname!='/' && *fname!='\\');
+        const char *fname = name + strlen(name);
+        do --fname; while(fname >= name && *fname!='/' && *fname!='\\');
         fname++;
-        defformatstring(meshname)("models/%s/%s.iqm", loadname, fname);
+        defformatstring(meshname, "%s/%s.iqm", name, fname);
         mdl.meshes = sharemeshes(path(meshname), NULL);
         if(!mdl.meshes) return false;
         mdl.initanimparts();
@@ -399,9 +393,8 @@ struct iqm : skelmodel, skelloader<iqm>
 
     bool load()
     {
-        if(loaded) return true;
-        formatstring(dir)("models/%s", loadname);
-        defformatstring(cfgname)("models/%s/iqm.cfg", loadname);
+        formatstring(dir, "%s", name);
+        defformatstring(cfgname, "%s/iqm.cfg", name);
 
         loading = this;
         if(execfile(cfgname, false) && parts.length()) // configured iqm, will call the iqm* commands below
@@ -418,15 +411,8 @@ struct iqm : skelmodel, skelloader<iqm>
             }
             loading = NULL;
         }
-        scale /= 4;
-        parts[0]->translate = translate;
-        loopv(parts) 
-        {
-            skelpart *p = (skelpart *)parts[i];
-            p->endanimparts();
-            p->meshes->shared++;
-        }
-        return loaded = true;
+        loaded();
+        return true;
     }
 };
 
