@@ -75,7 +75,10 @@ int lightmapping = 0;
 
 vector<LightMap> lightmaps;
 
-VAR(IDF_WORLD, lightprecision, 1, 32, 256);
+int curlightprecision = 32;
+VAR(IDF_WORLD, lightprecision, 1, 32, 2048);
+VAR(IDF_WORLD, lightprecisionquick, 1, 2048, 2048);
+
 VAR(IDF_WORLD, lighterror, 1, 8, 16);
 VAR(IDF_WORLD, bumperror, 1, 3, 16);
 VAR(IDF_WORLD, lightlod, 0, 0, 10);
@@ -211,7 +214,7 @@ void show_calclight_lmprog()
 {
     float bar1 = float(lmprog) / float(allocnodes);
 
-    defformatstring(text)("%d textures used", lightmaps.length());
+    defformatstring(text, "%d textures used", lightmaps.length());
 
     if(LM_PACKW <= hwtexsize && !lmprogtex)
     {
@@ -438,14 +441,14 @@ static void updatelightmap(const layoutinfo &surface)
         lm.offsetx = lm.offsety = 0;
         lm.tex = lightmaptexs.length();
         LightMapTexture &tex = lightmaptexs.add();
-        tex.type = renderpath==R_FIXEDFUNCTION ? (lm.type&~LM_TYPE) | LM_DIFFUSE : lm.type;
+        tex.type = lm.type;
         tex.w = LM_PACKW;
         tex.h = LM_PACKH;
         tex.unlitx = lm.unlitx;
         tex.unlity = lm.unlity;
         glGenTextures(1, &tex.id);
         createtexture(tex.id, tex.w, tex.h, NULL, 3, 1, tex.type&LM_ALPHA ? GL_RGBA : GL_RGB);
-        if(renderpath!=R_FIXEDFUNCTION && (lm.type&LM_TYPE)==LM_BUMPMAP0 && lightmaps.inrange(surface.lmid+1-LMID_RESERVED))
+        if((lm.type&LM_TYPE)==LM_BUMPMAP0 && lightmaps.inrange(surface.lmid+1-LMID_RESERVED))
         {
             LightMap &lm2 = lightmaps[surface.lmid+1-LMID_RESERVED];
             lm2.offsetx = lm2.offsety = 0;
@@ -466,7 +469,7 @@ static void updatelightmap(const layoutinfo &surface)
 
     glBindTexture(GL_TEXTURE_2D, lightmaptexs[lm.tex].id);
     glTexSubImage2D(GL_TEXTURE_2D, 0, lm.offsetx + surface.x, lm.offsety + surface.y, surface.w, surface.h, lm.type&LM_ALPHA ? GL_RGBA : GL_RGB, GL_UNSIGNED_BYTE, &lm.data[(surface.y*LM_PACKW + surface.x)*lm.bpp]);
-    if(renderpath!=R_FIXEDFUNCTION && (lm.type&LM_TYPE)==LM_BUMPMAP0 && lightmaps.inrange(surface.lmid+1-LMID_RESERVED))
+    if((lm.type&LM_TYPE)==LM_BUMPMAP0 && lightmaps.inrange(surface.lmid+1-LMID_RESERVED))
     {
         LightMap &lm2 = lightmaps[surface.lmid+1-LMID_RESERVED];
         glBindTexture(GL_TEXTURE_2D, lightmaptexs[lm2.tex].id);
@@ -637,7 +640,7 @@ void calcsunlight(lightmapworker *w, const vec &o, const vec &normal, float tole
             hit++;
         if(offset)
         {
-            matrix3x3 rot(90*RAD, dir);
+            matrix3 rot(90*RAD, dir);
             vec spoke(yaw*RAD, (pitch + offset)*RAD);
             spoke.rotate(21*RAD, dir);
             loopk(4)
@@ -679,8 +682,8 @@ static inline void generatealpha(lightmapworker *w, float tolerance, const vec &
         static const int sdim[] = { 1, 0, 0 }, tdim[] = { 2, 2, 1 };
         int dim = dimension(w->orient);
         float k = 8.0f/w->vslot->scale,
-              s = (pos[sdim[dim]] * k - w->vslot->xoffset) / w->slot->layermaskscale,
-              t = (pos[tdim[dim]] * (dim <= 1 ? -k : k) - w->vslot->yoffset) / w->slot->layermaskscale;
+              s = (pos[sdim[dim]] * k - w->vslot->offset.x) / w->slot->layermaskscale,
+              t = (pos[tdim[dim]] * (dim <= 1 ? -k : k) - w->vslot->offset.y) / w->slot->layermaskscale;
         if((w->rotate&5)==1) swap(s, t);
         if(w->rotate>=2 && w->rotate<=4) s = -s;
         if((w->rotate>=1 && w->rotate<=2) || w->rotate==5) t = -t;
@@ -1366,7 +1369,7 @@ static int setupsurface(lightmapworker *w, plane planes[2], int numplanes, const
     }
 
     int scale = int(min(cmax.x - cmin.x, cmax.y - cmin.y));
-    float lpu = 16.0f / float(lightlod && scale < (1 << lightlod) ? max(lightprecision / 2, 1) : lightprecision);
+    float lpu = 16.0f / float(lightlod && scale < (1 << lightlod) ? max(curlightprecision / 2, 1) : curlightprecision);
     int lw = clamp(int(ceil((cmax.x - cmin.x + 1)*lpu)), LM_MINW, LM_MAXW), lh = clamp(int(ceil((cmax.y - cmin.y + 1)*lpu)), LM_MINH, LM_MAXH);
     w->w = lw;
     w->h = lh;
@@ -1518,8 +1521,8 @@ static lightmapinfo *setupsurfaces(lightmapworker *w, lightmaptask &task)
             if(usefaces&2) curlitverts[numverts++].set(v[(order+3)&3].mul(size).add(vo));
         }
 
-        vec pos[MAXFACEVERTS], n[MAXFACEVERTS], po = ivec(co).mask(~0xFFF).tovec();
-        loopj(numverts) pos[j] = curlitverts[j].getxyz().tovec().mul(1.0f/8).add(po);
+        vec pos[MAXFACEVERTS], n[MAXFACEVERTS], po(ivec(co).mask(~0xFFF));
+        loopj(numverts) pos[j] = vec(curlitverts[j].getxyz()).mul(1.0f/8).add(po);
 
         plane planes[2];
         int numplanes = 0;
@@ -1744,7 +1747,7 @@ static bool processtasks(bool finish = false)
     return true;
 }
 
-static void generatelightmaps(cube *c, int cx, int cy, int cz, int size)
+static void generatelightmaps(cube *c, const ivec &co, int size)
 {
     CHECK_PROGRESS(return);
 
@@ -1752,9 +1755,9 @@ static void generatelightmaps(cube *c, int cx, int cy, int cz, int size)
 
     loopi(8)
     {
-        ivec o(i, cx, cy, cz, size);
+        ivec o(i, co, size);
         if(c[i].children)
-            generatelightmaps(c[i].children, o.x, o.y, o.z, size >> 1);
+            generatelightmaps(c[i].children, o, size >> 1);
         else if(!isempty(c[i]))
         {
             if(c[i].ext)
@@ -1769,7 +1772,7 @@ static void generatelightmaps(cube *c, int cx, int cy, int cz, int size)
             int usefacemask = 0;
             loopj(6) if(c[i].texture[j] != DEFAULT_SKY && (!(c[i].merged&(1<<j)) || (c[i].ext && c[i].ext->surfaces[j].numverts&MAXFACEVERTS)))
             {
-                usefacemask |= visibletris(c[i], j, o.x, o.y, o.z, size)<<(4*j);
+                usefacemask |= visibletris(c[i], j, o, size)<<(4*j);
             }
             if(usefacemask)
             {
@@ -1794,7 +1797,7 @@ static bool previewblends(lightmapworker *w, cube &c, const ivec &co, int size)
 
     int usefacemask = 0;
     loopi(6) if(c.texture[i] != DEFAULT_SKY && lookupvslot(c.texture[i], false).layer)
-        usefacemask |= visibletris(c, i, co.x, co.y, co.z, size)<<(4*i);
+        usefacemask |= visibletris(c, i, co, size)<<(4*i);
     if(!usefacemask) return false;
 
     if(!setblendmaporigin(w->blendmapcache, co, size))
@@ -1851,8 +1854,8 @@ static bool previewblends(lightmapworker *w, cube &c, const ivec &co, int size)
         curlitverts[numverts++].set(v[order+2].mul(size).add(vo));
         if(usefaces&2) curlitverts[numverts++].set(v[(order+3)&3].mul(size).add(vo));
 
-        vec pos[4], n[4], po = ivec(co).mask(~0xFFF).tovec();
-        loopj(numverts) pos[j] = curlitverts[j].getxyz().tovec().mul(1.0f/8).add(po);
+        vec pos[4], n[4], po(ivec(co).mask(~0xFFF));
+        loopj(numverts) pos[j] = vec(curlitverts[j].getxyz()).mul(1.0f/8).add(po);
 
         plane planes[2];
         int numplanes = 0;
@@ -1939,7 +1942,7 @@ static bool previewblends(lightmapworker *w, cube *c, const ivec &co, int size,
     bool changed = false;
     loopoctabox(co, size, bo, bs)
     {
-        ivec o(i, co.x, co.y, co.z, size);
+        ivec o(i, co, size);
         cubeext *ext = c[i].ext;
         if(ext && ext->va && ext->va->hasmerges)
         {
@@ -2038,7 +2041,7 @@ bool lightmapworker::setupthread()
 {
     if(!spacecond) spacecond = SDL_CreateCond();
     if(!spacecond) return false;
-    thread = SDL_CreateThread(work, this);
+    thread = SDL_CreateThread(work, "lightmap worker", this);
     return thread!=NULL;
 }
 
@@ -2048,8 +2051,9 @@ static Uint32 calclighttimer(Uint32 interval, void *param)
     return interval;
 }
 
-bool setlightmapquality(int quality)
+bool setlightmapquality(int quality, bool quick)
 {
+    curlightprecision = quick ? lightprecisionquick : lightprecision;
     switch(quality)
     {
         case  1: lmshadows = 2; lmaa = 3; lerptjoints = 1; break;
@@ -2129,9 +2133,9 @@ static void cleanupthreads()
     lightmapping = 0;
 }
 
-void calclight(int *quality)
+void calclight(int *quality, int *quick)
 {
-    if(!setlightmapquality(*quality))
+    if(!setlightmapquality(*quality, *quick!=0))
     {
         conoutft(CON_MESG, "\frvalid range for calclight quality is -1..1");
         return;
@@ -2155,7 +2159,7 @@ void calclight(int *quality)
     calcnormals(lerptjoints > 0);
     show_calclight_lmprog();
     setupthreads(numthreads);
-    generatelightmaps(worldroot, 0, 0, 0, hdr.worldsize >> 1);
+    generatelightmaps(worldroot, ivec(0, 0, 0), hdr.worldsize >> 1);
     cleanupthreads();
     clearnormals();
     Uint32 end = SDL_GetTicks();
@@ -2183,16 +2187,16 @@ void calclight(int *quality)
     if(lmprogtex) { glDeleteTextures(1, &lmprogtex); lmprogtex = 0; }
 }
 
-COMMAND(0, calclight, "i");
+COMMAND(0, calclight, "ii");
 
 VAR(0, patchnormals, 0, 0, 1);
 
-void patchlight(int *quality)
+void patchlight(int *quality, int *quick)
 {
     if(noedit(true)) return;
-    if(!setlightmapquality(*quality))
+    if(!setlightmapquality(*quality, *quick!=0))
     {
-        conoutft(CON_MESG, "\frvalid range for patchlight quality is 0..3");
+        conoutft(CON_MESG, "\frvalid range for patchlight quality is -1..1");
         return;
     }
     progress(0, "patching lightmaps...");
@@ -2219,7 +2223,7 @@ void patchlight(int *quality)
     if(patchnormals) calcnormals(lerptjoints > 0);
     show_calclight_lmprog();
     setupthreads(numthreads);
-    generatelightmaps(worldroot, 0, 0, 0, hdr.worldsize >> 1);
+    generatelightmaps(worldroot, ivec(0, 0, 0), hdr.worldsize >> 1);
     cleanupthreads();
     if(patchnormals) clearnormals();
     Uint32 end = SDL_GetTicks();
@@ -2243,7 +2247,7 @@ void patchlight(int *quality)
     if(lmprogtex) { glDeleteTextures(1, &lmprogtex); lmprogtex = 0; }
 }
 
-COMMAND(0, patchlight, "i");
+COMMAND(0, patchlight, "ii");
 
 void clearlightmaps()
 {
@@ -2341,7 +2345,7 @@ void fixrotatedlightmaps(cube &c, const ivec &co, int size)
 {
     if(c.children)
     {
-        loopi(8) fixrotatedlightmaps(c.children[i], ivec(i, co.x, co.y, co.z, size>>1), size>>1);
+        loopi(8) fixrotatedlightmaps(c.children[i], ivec(i, co, size>>1), size>>1);
         return;
     }
     if(!c.ext) return;
@@ -2352,7 +2356,7 @@ void fixrotatedlightmaps(cube &c, const ivec &co, int size)
         int numverts = surf.numverts&MAXFACEVERTS;
         if(numverts!=4 || (surf.lmid[0] < LMID_RESERVED && surf.lmid[1] < LMID_RESERVED)) continue;
         vertinfo *verts = c.ext->verts() + surf.verts;
-        int vis = visibletris(c, i, co.x, co.y, co.z, size);
+        int vis = visibletris(c, i, co, size);
         if(!vis || vis==3) continue;
         if((verts[0].u != verts[1].u || verts[0].v != verts[1].v) &&
            (verts[0].u != verts[3].u || verts[0].v != verts[3].v) &&
@@ -2386,32 +2390,7 @@ void fixrotatedlightmaps(cube &c, const ivec &co, int size)
 
 void fixrotatedlightmaps()
 {
-    loopi(8) fixrotatedlightmaps(worldroot[i], ivec(i, 0, 0, 0, hdr.worldsize>>1), hdr.worldsize>>1);
-}
-
-static void convertlightmap(LightMap &lmc, LightMap &lmlv, uchar *dst, size_t stride)
-{
-    const uchar *c = lmc.data;
-    const bvec *lv = (const bvec *)lmlv.data;
-    loopi(LM_PACKH)
-    {
-        uchar *dstrow = dst;
-        loopj(LM_PACKW)
-        {
-            int z = int(lv->z)*2 - 255,
-                r = (int(c[0]) * z) / 255,
-                g = (int(c[1]) * z) / 255,
-                b = (int(c[2]) * z) / 255;
-            dstrow[0] = max(r, int(ambientcolor[0]));
-            dstrow[1] = max(g, int(ambientcolor[1]));
-            dstrow[2] = max(b, int(ambientcolor[2]));
-            if(lmc.bpp==4) dstrow[3] = c[3];
-            c += lmc.bpp;
-            lv++;
-            dstrow += lmc.bpp;
-        }
-        dst += stride;
-    }
+    loopi(8) fixrotatedlightmaps(worldroot[i], ivec(i, ivec(0, 0, 0), hdr.worldsize>>1), hdr.worldsize>>1);
 }
 
 static void copylightmap(LightMap &lm, uchar *dst, size_t stride)
@@ -2425,14 +2404,12 @@ static void copylightmap(LightMap &lm, uchar *dst, size_t stride)
     }
 }
 
-VARF(0, convertlms, 0, 1, 1, { cleanuplightmaps(); initlights(); allchanged(); });
-
 void genreservedlightmaptexs()
 {
     while(lightmaptexs.length() < LMID_RESERVED)
     {
         LightMapTexture &tex = lightmaptexs.add();
-        tex.type = renderpath != R_FIXEDFUNCTION && lightmaptexs.length()&1 ? LM_DIFFUSE : LM_BUMPMAP1;
+        tex.type = lightmaptexs.length()&1 ? LM_DIFFUSE : LM_BUMPMAP1;
         glGenTextures(1, &tex.id);
     }
     uchar unlit[3] = { ambientcolor[0], ambientcolor[1], ambientcolor[2] };
@@ -2498,12 +2475,6 @@ void genlightmaptexs(int flagmask, int flagval)
         if(lm.unlitx < 0) findunlit(i);
     }
 
-    if(renderpath==R_FIXEDFUNCTION)
-    {
-        remaining[LM_DIFFUSE] += remaining[LM_BUMPMAP0];
-        remaining[LM_BUMPMAP0] = remaining[LM_BUMPMAP1] = 0;
-    }
-
     int sizelimit = (maxtexsize ? min(maxtexsize, hwtexsize) : hwtexsize)/max(LM_PACKW, LM_PACKH);
     sizelimit = min(batchlightmaps, sizelimit*sizelimit);
     while(total)
@@ -2514,8 +2485,7 @@ void genlightmaptexs(int flagmask, int flagval)
         {
             LightMap &lm = lightmaps[i];
             if(lm.tex >= 0 || (lm.type&flagmask) != flagval) continue;
-            if(renderpath != R_FIXEDFUNCTION) type = lm.type&LM_TYPE;
-            else if((lm.type&LM_TYPE) == LM_BUMPMAP1) continue;
+            type = lm.type&LM_TYPE;
             firstlm = &lm;
             break;
         }
@@ -2536,18 +2506,12 @@ void genlightmaptexs(int flagmask, int flagval)
         tex.w = LM_PACKW<<((used+1)/2);
         tex.h = LM_PACKH<<(used/2);
         int bpp = firstlm->bpp;
-        uchar *data = used || (renderpath == R_FIXEDFUNCTION && (firstlm->type&LM_TYPE) == LM_BUMPMAP0 && convertlms) ?
-            new uchar[bpp*tex.w*tex.h] :
-            NULL;
+        uchar *data = used ? new uchar[bpp*tex.w*tex.h] : NULL;
         int offsetx = 0, offsety = 0;
         loopv(lightmaps)
         {
             LightMap &lm = lightmaps[i];
-            if(lm.tex >= 0 || (lm.type&flagmask) != flagval ||
-               (renderpath==R_FIXEDFUNCTION ?
-                (lm.type&LM_TYPE) == LM_BUMPMAP1 :
-                (lm.type&LM_TYPE) != type))
-                continue;
+            if(lm.tex >= 0 || (lm.type&flagmask) != flagval || (lm.type&LM_TYPE) != type) continue;
 
             lm.tex = lightmaptexs.length()-1;
             lm.offsetx = offsetx;
@@ -2558,12 +2522,7 @@ void genlightmaptexs(int flagmask, int flagval)
                 tex.unlity = offsety + lm.unlity;
             }
 
-            if(data)
-            {
-                if(renderpath == R_FIXEDFUNCTION && (lm.type&LM_TYPE) == LM_BUMPMAP0 && convertlms)
-                    convertlightmap(lm, lightmaps[i+1], &data[bpp*(offsety*tex.w + offsetx)], bpp*tex.w);
-                else copylightmap(lm, &data[bpp*(offsety*tex.w + offsetx)], bpp*tex.w);
-            }
+            if(data) copylightmap(lm, &data[bpp*(offsety*tex.w + offsetx)], bpp*tex.w);
 
             offsetx += LM_PACKW;
             if(offsetx >= tex.w) { offsetx = 0; offsety += LM_PACKH; }
@@ -2589,7 +2548,6 @@ void clearlights()
         e.light.dir = vec(0, 0, 1);
     }
     shouldlightents = false;
-    if(nolights) return;
 
     genlightmaptexs(LM_ALPHA, 0);
     genlightmaptexs(LM_ALPHA, LM_ALPHA);
@@ -2622,7 +2580,7 @@ void lightents(bool force)
 
 void initlights()
 {
-    if(nolights || (fullbright && editmode) || lightmaps.empty())
+    if((fullbright && editmode) || lightmaps.empty())
     {
         clearlights();
         return;
@@ -2632,7 +2590,7 @@ void initlights()
     genlightmaptexs(LM_ALPHA, 0);
     genlightmaptexs(LM_ALPHA, LM_ALPHA);
     brightengeom = false;
-    shouldlightents = true; 
+    shouldlightents = true;
 }
 
 static inline void fastskylight(const vec &o, float tolerance, uchar *skylight, int flags = RAY_ALPHAPOLY, extentity *t = NULL, bool fast = false)
@@ -2681,7 +2639,7 @@ static inline void fastsunlight(const vec &o, float tolerance, uchar *slight, in
 
 void lightreaching(const vec &target, vec &color, vec &dir, bool fast, extentity *t, float ambient)
 {
-    if(nolights || (fullbright && editmode) || lightmaps.empty())
+    if((fullbright && editmode) || lightmaps.empty())
     {
         color = vec(1, 1, 1);
         dir = vec(0, 0, 1);
@@ -2827,7 +2785,7 @@ void dumplms()
     loopv(lightmaps)
     {
         ImageData temp(LM_PACKW, LM_PACKH, lightmaps[i].bpp, lightmaps[i].data);
-        defformatstring(fname)("%s.lm%.4d", mapname, i);
+        defformatstring(fname, "%s.lm%.4d", mapname, i);
         saveimage(fname, temp, imageformat, compresslevel, true);
     }
 }
