@@ -105,8 +105,7 @@ struct md2 : vertmodel, vertloader<md2>
                         idx = &tchash[tckey];
                         *idx = tcverts.length();
                         tcvert &tc = tcverts.add();
-                        tc.u = u.f;
-                        tc.v = v.f;
+                        tc.tc = vec2(u.f, v.f);
                         vindexes.add((ushort)vindex);
                     }
                     idxs.add(*idx);
@@ -210,37 +209,31 @@ struct md2 : vertmodel, vertloader<md2>
 
     bool load()
     {
-        if(loaded) return true;
-        part &mdl = *new part;
-        parts.add(&mdl);
-        mdl.model = this;
-        mdl.index = 0;
-        const char *pname = parentdir(loadname);
-        defformatstring(name1)("models/%s/tris.md2", loadname);
+        part &mdl = addpart();
+        const char *pname = parentdir(name);
+        defformatstring(name1, "%s/tris.md2", name);
         mdl.meshes = sharemeshes(path(name1));
         if(!mdl.meshes)
         {
-            defformatstring(name2)("models/%s/tris.md2", pname);    // try md2 in parent folder (vert sharing)
+            defformatstring(name2, "%s/tris.md2", pname);    // try md2 in parent folder (vert sharing)
             mdl.meshes = sharemeshes(path(name2));
             if(!mdl.meshes) return false;
         }
         Texture *tex, *masks;
-        loadskin(loadname, pname, tex, masks);
+        loadskin(name, pname, tex, masks);
         mdl.initskins(tex, masks);
         if(tex==notexture) conoutf("\frcould not load model skin for %s", name1);
         loading = this;
-        defformatstring(name3)("models/%s/md2.cfg", loadname);
+        defformatstring(name3, "%s/md2.cfg", name);
         if(!execfile(name3, false))
         {
-            formatstring(name3)("models/%s/md2.cfg", pname);
+            formatstring(name3, "%s/md2.cfg", pname);
             execfile(name3, false);
         }
         loading = 0;
-        scale /= 4;
         translate.y = -translate.y;
-        parts[0]->translate = translate;
-        loopv(parts) parts[i]->meshes->shared++;
-        return loaded = true;
+        loaded();
+        return true;
     }
 };
 
