@@ -2,6 +2,7 @@ enum { MDL_MD2 = 0, MDL_MD3, MDL_MD5, MDL_OBJ, MDL_SMD, MDL_IQM, NUMMODELTYPES }
 
 struct model
 {
+    char *name;
     float spinyaw, spinpitch, spinroll, offsetyaw, offsetpitch, offsetroll;
     bool collide, ellipsecollide, shadow, alphadepth, depthoffset;
     float scale;
@@ -11,12 +12,11 @@ struct model
     float rejectradius, height, collidexyradius, collideheight;
     int batch;
 
-    model() : spinyaw(0), spinpitch(0), spinroll(0), offsetyaw(0), offsetpitch(0), offsetroll(0), collide(true), ellipsecollide(false), shadow(true), alphadepth(true), depthoffset(false), scale(1.0f), translate(0, 0, 0), bih(0), bbcenter(0, 0, 0), bbradius(-1, -1, -1), bbextend(0, 0, 0), collidecenter(0, 0, 0), collideradius(-1, -1, -1), rejectradius(-1), height(0.45f), collidexyradius(0), collideheight(0), batch(-1) {}
-    virtual ~model() { DELETEP(bih); }
+    model(const char *name) : name(name ? newstring(name) : NULL), spinyaw(0), spinpitch(0), spinroll(0), offsetyaw(0), offsetpitch(0), offsetroll(0), collide(true), ellipsecollide(false), shadow(true), alphadepth(true), depthoffset(false), scale(1.0f), translate(0, 0, 0), bih(0), bbcenter(0, 0, 0), bbradius(-1, -1, -1), bbextend(0, 0, 0), collidecenter(0, 0, 0), collideradius(-1, -1, -1), rejectradius(-1), height(0.45f), collidexyradius(0), collideheight(0), batch(-1) {}
+    virtual ~model() { DELETEA(name); DELETEP(bih); }
     virtual void calcbb(vec &center, vec &radius) = 0;
     virtual void render(int anim, int basetime, int basetime2, const vec &o, float yaw, float pitch, float roll, dynent *d, modelattach *a = NULL, const vec &color = vec(0, 0, 0), const vec &dir = vec(0, 0, 0), const bvec *material = NULL, float transparent = 1, float size = 1) = 0;
     virtual bool load() = 0;
-    virtual const char *name() const = 0;
     virtual int type() const = 0;
     virtual BIH *setBIH() { return 0; }
     virtual bool envmapped() { return false; }
@@ -35,7 +35,7 @@ struct model
     virtual void setmaterial(int material, int material2) {}
 
     virtual void preloadBIH() { if(!bih) setBIH(); }
-    virtual void preloadshaders() {}
+    virtual void preloadshaders(bool force = false) {}
     virtual void preloadmeshes() {}
     virtual void cleanup() {}
 
