@@ -17,7 +17,7 @@ static inline clipplanes &getclipplanes(const cube &c, const ivec &o, int size,
     {
         p.owner = &c;
         p.version = clipcacheversion+offset;
-        genclipplanes(c, o.x, o.y, o.z, size, p, collide);
+        genclipplanes(c, o, size, p, collide);
     }
     return p;
 }
@@ -427,7 +427,7 @@ float shadowray(ShadowRayCache *cache, const vec &o, const vec &ray, float radiu
         {
             if(isentirelysolid(c)) return c.texture[side]==DEFAULT_SKY && mode&RAY_SKIPSKY ? radius : dist;
             clipplanes &p = cache->clipcache[int(&c - worldroot)&(MAXCLIPPLANES-1)];
-            if(p.owner != &c || p.version != cache->version) { p.owner = &c; p.version = cache->version; genclipplanes(c, lo.x, lo.y, lo.z, 1<<lshift, p, false); }
+            if(p.owner != &c || p.version != cache->version) { p.owner = &c; p.version = cache->version; genclipplanes(c, lo, 1<<lshift, p, false); }
             INTERSECTPLANES(side = p.side[i], goto nextcube);
             INTERSECTBOX(side = (i<<1) + 1 - lsizemask[i], goto nextcube);
             if(exitdist >= 0) return c.texture[side]==DEFAULT_SKY && mode&RAY_SKIPSKY ? radius : dist+max(enterdist+0.1f, 0.0f);
@@ -490,7 +490,7 @@ float rayfloor(const vec &o, vec &floor, int mode, float radius)
 /////////////////////////  entity collision  ///////////////////////////////////////////////
 
 // info about collisions
-bool collideinside; // whether an internal collision happened
+int collideinside; // whether an internal collision happened
 physent *hitplayer; // whether the collection hit a player
 int hitflags = HITFLAG_NONE;
 vec collidewall; // just the normal vectors.
@@ -545,7 +545,7 @@ bool ellipseboxcollide(physent *d, const vec &dir, const vec &o, const vec &cent
             collidewall = vec(0, 0, 1);
             return true;
         }
-        collideinside = true;
+        collideinside++;
     }
     return false;
 }
@@ -584,7 +584,7 @@ bool ellipsecollide(physent *d, const vec &dir, const vec &o, const vec &center,
             collidewall = vec(0, 0, 1);
             return true;
         }
-        collideinside = true;
+        collideinside++;
     }
     return false;
 }
@@ -675,7 +675,7 @@ static inline bool plcollide(physent *d, const vec &dir, physent *o)
         vec wn = vec(cp).sub(obvol.center());
         collidewall = obvol.contactface(wn, dir.iszero() ? vec(wn).neg() : dir);
         if(!collidewall.iszero()) return true;
-        collideinside = true;
+        collideinside++;
     }
     return false;
 }
@@ -696,9 +696,11 @@ bool plcollide(physent *d, const vec &dir, physent *o)
     }
 }
 
-bool plcollide(physent *d, const vec &dir)  // collide with player or monster
+bool plcollide(physent *d, const vec &dir, bool insideplayercol)  // collide with player or monster
 {
     if(d->type==ENT_CAMERA || d->state!=CS_ALIVE) return false;
+    int lastinside = collideinside;
+    physent *insideplayer = NULL;
     loopdynentcache(x, y, d->o, d->radius)
     {
         const vector<physent *> &dynents = checkdynentcache(x, y);
@@ -711,14 +713,24 @@ bool plcollide(physent *d, const vec &dir)  // collide with player or monster
                 hitplayer = o;
                 return true;
             }
+            if(collideinside > lastinside)
+            {   
+                lastinside = collideinside;
+                insideplayer = o;
+            }
         }
     }
+    if(insideplayer && insideplayercol)
+    {
+        hitplayer = insideplayer;
+        return true;
+    }
     return false;
 }
 
 void rotatebb(vec &center, vec &radius, int yaw, int pitch, int roll)
 {
-    matrix3x3 orient;
+    matrix3 orient;
     orient.identity();
     if(yaw) orient.rotate_around_z(sincosmod360(yaw));
     if(roll) orient.rotate_around_x(sincosmod360(-roll));
@@ -738,7 +750,7 @@ static inline bool mmcollide(physent *d, const vec &dir, const extentity &e, con
         vec wn = vec(cp).sub(mdlvol.center());
         collidewall = mdlvol.contactface(wn, dir.iszero() ? vec(wn).neg() : dir);
         if(!collidewall.iszero()) return true;
-        collideinside = true;
+        collideinside++;
     }
     return false;
 }
@@ -763,12 +775,12 @@ static bool fuzzycollidebox(physent *d, const vec &dir, float cutoff, const vec
         switch(i)
         {
             default:
-            case 0: w = vec(mdlvol.orient.a).neg(); dist = -radius.x; break;
-            case 1: w = mdlvol.orient.a; dist = -radius.x; break;
-            case 2: w = vec(mdlvol.orient.b).neg(); dist = -radius.y; break;
-            case 3: w = mdlvol.orient.b; dist = -radius.y; break;
-            case 4: w = vec(mdlvol.orient.c).neg(); dist = -radius.z; break;
-            case 5: w = mdlvol.orient.c; dist = -radius.z; break;
+            case 0: w = mdlvol.orient.rowx().neg(); dist = -radius.x; break;
+            case 1: w = mdlvol.orient.rowx(); dist = -radius.x; break;
+            case 2: w = mdlvol.orient.rowy().neg(); dist = -radius.y; break;
+            case 3: w = mdlvol.orient.rowy(); dist = -radius.y; break;
+            case 4: w = mdlvol.orient.rowz().neg(); dist = -radius.z; break;
+            case 5: w = mdlvol.orient.rowz(); dist = -radius.z; break;
         }
         vec pw = entvol.supportpoint(vec(w).neg());
         dist += w.dot(vec(pw).sub(mdlvol.o));
@@ -789,7 +801,7 @@ static bool fuzzycollidebox(physent *d, const vec &dir, float cutoff, const vec
     }
     if(collidewall.iszero())
     {
-        collideinside = true;
+        collideinside++;
         return false;
     }
     return true;
@@ -815,8 +827,8 @@ static bool fuzzycollideellipse(physent *d, const vec &dir, float cutoff, const
         switch(i)
         {
             default:
-            case 0: w = mdlvol.orient.c; dist = -radius.z; break;
-            case 1: w = vec(mdlvol.orient.c).neg(); dist = -radius.z; break;
+            case 0: w = mdlvol.orient.rowz(); dist = -radius.z; break;
+            case 1: w = mdlvol.orient.rowz().neg(); dist = -radius.z; break;
             case 2:
             {
                 vec2 ln(mdlvol.orient.transform(entvol.center().sub(mdlvol.o)));
@@ -847,7 +859,7 @@ static bool fuzzycollideellipse(physent *d, const vec &dir, float cutoff, const
     }
     if(collidewall.iszero())
     {
-        collideinside = true;
+        collideinside++;
         return false;
     }
     return true;
@@ -936,7 +948,7 @@ static bool fuzzycollidesolid(physent *d, const vec &dir, float cutoff, const cu
 
     if(collidewall.iszero())
     {
-        collideinside = true;
+        collideinside++;
         return false;
     }
     return true;
@@ -1014,7 +1026,7 @@ static bool fuzzycollideplanes(physent *d, const vec &dir, float cutoff, const c
     if(bestplane >= 0) collidewall = p.p[bestplane];
     else if(collidewall.iszero())
     {
-        collideinside = true;
+        collideinside++;
         return false;
     }
     return true;
@@ -1044,7 +1056,7 @@ static bool cubecollidesolid(physent *d, const vec &dir, float cutoff, const cub
 
     if(collidewall.iszero())
     {
-        collideinside = true;
+        collideinside++;
         return false;
     }
     return true;
@@ -1097,7 +1109,7 @@ static bool cubecollideplanes(physent *d, const vec &dir, float cutoff, const cu
     if(bestplane >= 0) collidewall = p.p[bestplane];
     else if(collidewall.iszero())
     {
-        collideinside = true;
+        collideinside++;
         return false;
     }
     return true;
@@ -1125,7 +1137,7 @@ static inline bool octacollide(physent *d, const vec &dir, float cutoff, const i
     loopoctabox(cor, size, bo, bs)
     {
         if(c[i].ext && c[i].ext->ents) if(mmcollide(d, dir, cutoff, *c[i].ext->ents)) return true;
-        ivec o(i, cor.x, cor.y, cor.z, size);
+        ivec o(i, cor, size);
         if(c[i].children)
         {
             if(octacollide(d, dir, cutoff, bo, bs, c[i].children, o, size>>1)) return true;
@@ -1173,16 +1185,16 @@ static inline bool octacollide(physent *d, const vec &dir, float cutoff, const i
 }
 
 // all collision happens here
-bool collide(physent *d, const vec &dir, float cutoff, bool playercol)
+bool collide(physent *d, const vec &dir, float cutoff, bool playercol, bool insideplayercol)
 {
-    collideinside = false;
+    collideinside = 0;
     hitplayer = NULL;
     hitflags = HITFLAG_NONE;
     collidewall = vec(0, 0, 0);
     ivec bo(int(d->o.x-d->radius), int(d->o.y-d->radius), int(d->o.z-d->height)),
          bs(int(d->o.x+d->radius), int(d->o.y+d->radius), int(d->o.z+d->aboveeye));
     bs.add(1);  // guard space for rounding errors
-    return octacollide(d, dir, cutoff, bo, bs) || (playercol && plcollide(d, dir));
+    return octacollide(d, dir, cutoff, bo, bs) || (playercol && plcollide(d, dir, insideplayercol));
 }
 
 float pltracecollide(physent *d, const vec &from, const vec &ray, float maxdist)
@@ -1228,7 +1240,7 @@ void phystest()
 {
     static const char * const states[] = {"float", "fall", "slide", "slope", "floor", "step up", "step down", "bounce"};
     physent *player = (physent *)game::focusedent();
-    conoutf("PHYS(pl): %s, air %d, mat: %d, ladder: %s, floor: (%f, %f, %f), vel: (%f, %f, %f), g: (%f, %f, %f)", states[player->physstate], lastmillis-player->airmillis, player->inmaterial, player->onladder ? "yes" : "no", player->floor.x, player->floor.y, player->floor.z, player->vel.x, player->vel.y, player->vel.z, player->falling.x, player->falling.y, player->falling.z);
+    conoutf("PHYS(pl): %d %s, air %d, mat: %d, ladder: %s, floor: (%f, %f, %f), vel: (%f, %f, %f), g: (%f, %f, %f)", player->state, states[player->physstate], lastmillis-player->airmillis, player->inmaterial, player->onladder ? "yes" : "no", player->floor.x, player->floor.y, player->floor.z, player->vel.x, player->vel.y, player->vel.z, player->falling.x, player->falling.y, player->falling.z);
 }
 
 COMMAND(0, phystest, "");
