@@ -72,7 +72,7 @@ static void genpvsnodes(cube *c, int parent = 0, const ivec &co = ivec(0, 0, 0),
     int index = origpvsnodes.length();
     loopi(8)
     {
-        ivec o(i, co.x, co.y, co.z, size);
+        ivec o(i, co, size);
         pvsnode &n = origpvsnodes.add();
         n.flags = 0;
         n.children = 0;
@@ -98,7 +98,7 @@ static void genpvsnodes(cube *c, int parent = 0, const ivec &co = ivec(0, 0, 0),
     int branches = 0;
     loopi(8) if(c[i].children)
     {
-        ivec o(i, co.x, co.y, co.z, size);
+        ivec o(i, co, size);
         genpvsnodes(c[i].children, index+i, o, size>>1);
         if(origpvsnodes[index+i].children) branches++;
     }
@@ -112,20 +112,6 @@ struct shaftplane
     uchar rnear, cnear, rfar, cfar;
 };
 
-struct usvec
-{
-    union
-    {
-        struct { ushort x, y, z; };
-        ushort v[3];
-    };
-
-    ushort &operator[](int i) { return v[i]; }
-    ushort operator[](int i) const { return v[i]; }
-
-    ivec toivec() const { return ivec(x, y, z); }
-};
-
 struct shaftbb
 {
     union
@@ -409,7 +395,7 @@ struct pvsworker
             uchar flags = 0xFF;
             loopi(8)
             {
-                ivec o(i, co.x, co.y, co.z, size>>1);
+                ivec o(i, co, size>>1);
                 shaftcullpvs(s, children[i], o, size>>1);
                 flags &= children[i].flags;
             }
@@ -438,7 +424,7 @@ struct pvsworker
         {
             pvsnode *children = &pvsnodes[p.children];
             int csize = size>>1;
-            ivec dmin = ivec(co).add(csize>>1).sub(viewcellbb.min.toivec().add(viewcellbb.max.toivec()).shr(1)), dmax = ivec(dmin).add(csize);
+            ivec dmin = ivec(co).add(csize>>1).sub(ivec(viewcellbb.min).add(ivec(viewcellbb.max)).shr(1)), dmax = ivec(dmin).add(csize);
             dmin.mul(dmin);
             dmax.mul(dmax);
             ivec diff = ivec(dmax).sub(dmin);
@@ -459,13 +445,16 @@ struct pvsworker
                       dxz(order[1].index|order[3].index, order[1].dist+order[3].dist),
                       dyz(order[2].index|order[3].index, order[2].dist+order[3].dist);
             int j;
-            for(j = 4; j > 0 && dxy.dist < order[j-1].dist; --j) order[j] = order[j-1]; order[j] = dxy;
-            for(j = 5; j > 0 && dxz.dist < order[j-1].dist; --j) order[j] = order[j-1]; order[j] = dxz;
-            for(j = 6; j > 0 && dyz.dist < order[j-1].dist; --j) order[j] = order[j-1]; order[j] = dyz;
+            for(j = 4; j > 0 && dxy.dist < order[j-1].dist; --j) order[j] = order[j-1];
+            order[j] = dxy;
+            for(j = 5; j > 0 && dxz.dist < order[j-1].dist; --j) order[j] = order[j-1];
+            order[j] = dxz;
+            for(j = 6; j > 0 && dyz.dist < order[j-1].dist; --j) order[j] = order[j-1];
+            order[j] = dyz;
             loopi(8)
             {
                 int index = order[i].index^dir;
-                ivec o(index, co.x, co.y, co.z, csize);
+                ivec o(index, co, csize);
                 cullpvs(children[index], o, csize);
             }
             if(!(p.flags & PVS_HIDE_BB)) return;
@@ -473,7 +462,7 @@ struct pvsworker
         bvec edges = p.children ? bvec(0x80, 0x80, 0x80) : p.edges;
         if(edges.x==0xFF) return;
         shaftbb geom(co, size, edges);
-        ivec diff = geom.max.toivec().sub(viewcellbb.min.toivec()).abs();
+        ivec diff = ivec(geom.max).sub(ivec(viewcellbb.min)).abs();
         cullorder order[3] = { cullorder(0, diff.x), cullorder(1, diff.y), cullorder(2, diff.z) };
         if(order[1].dist > order[0].dist) swap(order[0], order[1]);
         if(order[2].dist > order[1].dist) swap(order[1], order[2]);
@@ -722,7 +711,7 @@ struct pvsworker
         pvsnode *children = &pvsnodes[p.children];
         loopoctabox(co, size, bbmin, bbmax)
         {
-            ivec o(i, co.x, co.y, co.z, size);
+            ivec o(i, co, size);
             if(children[i].flags & PVS_HIDE_BB) continue;
             if(!children[i].children || !materialoccluded(children[i], o, size/2, bbmin, bbmax)) return false;
         }
@@ -741,7 +730,7 @@ struct pvsworker
             bbmin[dim] += dimcoord(m.orient) ? -2 : 2;
             bbmax[C[dim]] += m.csize;
             bbmax[R[dim]] += m.rsize;
-            if(!materialoccluded(pvsnodes[0], vec(0, 0, 0), hdr.worldsize/2, bbmin, bbmax)) return false;
+            if(!materialoccluded(pvsnodes[0], ivec(0, 0, 0), hdr.worldsize/2, bbmin, bbmax)) return false;
         }
         return true;
     }
@@ -861,7 +850,7 @@ static void show_genpvs_progress(int unique = pvs.length(), int processed = numv
 {
     float bar1 = float(processed) / float(totalviewcells>0 ? totalviewcells : 1);
 
-    defformatstring(text1)("%d of %d view cells (%d unique)", processed, totalviewcells, unique);
+    defformatstring(text1, "%d of %d view cells (%d unique)", processed, totalviewcells, unique);
 
     progress(bar1, text1);
 
@@ -875,7 +864,6 @@ static void calcpvsbounds()
 {
     loopk(3) pvsbounds.min[k] = USHRT_MAX;
     loopk(3) pvsbounds.max[k] = 0;
-    extern vector<vtxarray *> valist;
     loopv(valist)
     {
         vtxarray *va = valist[i];
@@ -904,7 +892,7 @@ static int countviewcells(cube *c, const ivec &co, int size, int threshold)
     int count = 0;
     loopi(8)
     {
-        ivec o(i, co.x, co.y, co.z, size);
+        ivec o(i, co, size);
         if(pvsbounds.outside(o, size)) continue;
         cube &h = c[i];
         if(h.children)
@@ -927,7 +915,7 @@ static void genviewcells(viewcellnode &p, cube *c, const ivec &co, int size, int
     if(genpvs_canceled) return;
     loopi(8)
     {
-        ivec o(i, co.x, co.y, co.z, size);
+        ivec o(i, co, size);
         if(pvsbounds.outside(o, size)) continue;
         cube &h = c[i];
         if(h.children)
@@ -1036,7 +1024,6 @@ COMMAND(0, clearpvs, "");
 
 static void findwaterplanes()
 {
-    extern vector<vtxarray *> valist;
     loopi(MAXWATERPVS)
     {
         waterplanes[i].height = -1;
@@ -1095,8 +1082,7 @@ void testpvs(int *vcsize)
     int size = *vcsize>0 ? *vcsize : 32;
     for(int mask = 1; mask < size; mask <<= 1) size &= ~mask;
 
-    ivec o = camera1->o;
-    o.mask(~(size-1));
+    ivec o = ivec(camera1->o).mask(~(size-1));
     pvsworker w;
     int len;
     lockedpvs = w.testviewcell(o, size, &lockedwaterpvs, &len);
@@ -1138,7 +1124,7 @@ void genpvs(int *viewcellsize)
     numviewcells = 0;
     genpvs_canceled = false;
     check_genpvs_progress = false;
-    SDL_TimerID timer = NULL;
+    SDL_TimerID timer = 0;
     int numthreads = pvsthreads > 0 ? pvsthreads : numcpus;
     if(numthreads<=1)
     {
@@ -1159,7 +1145,7 @@ void genpvs(int *viewcellsize)
         loopi(numthreads)
         {
             pvsworker *w = pvsworkers.add(new pvsworker);
-            w->thread = SDL_CreateThread(pvsworker::run, w);
+            w->thread = SDL_CreateThread(pvsworker::run, "pvs worker", w);
         }
         show_genpvs_progress(0, 0);
         while(!genpvs_canceled)
@@ -1206,7 +1192,7 @@ static inline bool pvsoccluded(uchar *buf, const ivec &co, int size, const ivec
     uchar leafmask = buf[0];
     loopoctabox(co, size, bbmin, bbmax)
     {
-        ivec o(i, co.x, co.y, co.z, size);
+        ivec o(i, co, size);
         if(leafmask&(1<<i))
         {
             uchar leafvalues = buf[1+i];
@@ -1246,7 +1232,7 @@ bool pvsoccluded(const ivec &bbmin, const ivec &bbmax)
 bool pvsoccludedsphere(const vec &center, float radius)
 {
     if(curpvs==NULL) return false;
-    ivec bbmin = vec(center).sub(radius), bbmax = vec(center).add(radius+1);
+    ivec bbmin(vec(center).sub(radius)), bbmax(vec(center).add(radius+1));
     return pvsoccluded(curpvs, bbmin, bbmax);
 }
 
