@@ -260,7 +260,7 @@ COMMAND(0, mdlextendbb, "fff");
 void mdlname()
 {
     checkmdl;
-    result(loadingmodel->name());
+    result(loadingmodel->name);
 }
 
 COMMAND(0, mdlname, "");
@@ -334,7 +334,7 @@ void rdlimitrot(int *t1, int *t2, float *maxangle, float *qx, float *qy, float *
     r.tri[0] = *t1;
     r.tri[1] = *t2;
     r.maxangle = *maxangle * RAD;
-    r.middle = matrix3x3(quat(*qx, *qy, *qz, *qw));
+    r.middle = matrix3(quat(*qx, *qy, *qz, *qw));
 }
 COMMAND(0, rdlimitrot, "iifffff");
 
@@ -379,12 +379,12 @@ ICOMMAND(0, mapmodelindex, "s", (char *a), {
 
 // model registry
 
-hashtable<const char *, model *> mdllookup;
+hashnameset<model *> models;
 vector<const char *> preloadmodels;
 
 void preloadmodel(const char *name)
 {
-    if(!name || !name[0] || mdllookup.access(name)) return;
+    if(!name || !name[0] || models.access(name)) return;
     preloadmodels.add(newstring(name));
 }
 
@@ -442,7 +442,7 @@ model *loadmodel(const char *name, int i, bool msg)
         if(mmi.m) return mmi.m;
         name = mmi.name;
     }
-    model **mm = mdllookup.access(name);
+    model **mm = models.access(name);
     model *m;
     if(mm) m = *mm;
     else
@@ -450,7 +450,7 @@ model *loadmodel(const char *name, int i, bool msg)
         if(loadingmodel || lightmapping > 1) return NULL;
         if(msg)
         {
-            defformatstring(str)("models/%s", name);
+            defformatstring(str, "%s", name);
             progress(loadprogress, str);
         }
         loopi(NUMMODELTYPES)
@@ -463,35 +463,35 @@ model *loadmodel(const char *name, int i, bool msg)
         }
         loadingmodel = NULL;
         if(!m) return NULL;
-        mdllookup.access(m->name(), m);
+        models.access(m->name, m);
         m->preloadshaders();
     }
     if(mapmodels.inrange(i) && !mapmodels[i].m) mapmodels[i].m = m;
     return m;
 }
 
-void preloadmodelshaders()
+void preloadmodelshaders(bool force)
 {
     if(initing) return;
-    enumerate(mdllookup, model *, m, m->preloadshaders());
+    enumerate(models, model *, m, m->preloadshaders(force));
 }
 
 void clear_mdls()
 {
-    enumerate(mdllookup, model *, m, delete m);
+    enumerate(models, model *, m, delete m);
 }
 
 void cleanupmodels()
 {
-    enumerate(mdllookup, model *, m, m->cleanup());
+    enumerate(models, model *, m, m->cleanup());
 }
 
 void clearmodel(char *name)
 {
-    model **m = mdllookup.access(name);
+    model **m = models.access(name);
     if(!m) { conoutf("\frmodel %s is not loaded", name); return; }
     loopv(mapmodels) if(mapmodels[i].m==*m) mapmodels[i].m = NULL;
-    mdllookup.remove(name);
+    models.remove(name);
     (*m)->cleanup();
     delete *m;
     conoutf("\fgcleared model %s", name);
@@ -501,51 +501,48 @@ COMMAND(0, clearmodel, "s");
 
 bool modeloccluded(const vec &center, float radius)
 {
-    ivec bbmin = vec(center).sub(radius), bbmax = vec(center).add(radius+1);
+    ivec bbmin(vec(center).sub(radius)), bbmax(vec(center).add(radius+1));
     return pvsoccluded(bbmin, bbmax) || bboccluded(bbmin, bbmax);
 }
 
 VAR(0, showboundingbox, 0, 0, 2);
 
-void render2dbox(vec &o, float x, float y, float z)
+void render2dbox(vec &o, float x, float y, float z, const matrix4x3 *m)
 {
-    glBegin(GL_LINE_LOOP);
-    glVertex3f(o.x, o.y, o.z);
-    glVertex3f(o.x, o.y, o.z+z);
-    glVertex3f(o.x+x, o.y+y, o.z+z);
-    glVertex3f(o.x+x, o.y+y, o.z);
-    glEnd();
+    vec v[4] = { o, vec(o.x, o.y, o.z+z), vec(o.x+x, o.y+y, o.z+z), vec(o.x+x, o.y+y, o.z) };
+    gle::begin(GL_LINE_LOOP);
+    if(m) loopk(4) gle::attrib(m->transform(v[k]));
+    else loopk(4) gle::attrib(v[k]);
+    xtraverts += gle::end();
 }
 
-void render3dbox(vec &o, float tofloor, float toceil, float xradius, float yradius)
+void render3dbox(vec &o, float tofloor, float toceil, float xradius, float yradius, const matrix4x3 *m)
 {
     if(yradius<=0) yradius = xradius;
     vec c = o;
     c.sub(vec(xradius, yradius, tofloor));
     float xsz = xradius*2, ysz = yradius*2;
     float h = tofloor+toceil;
-    glColor3f(1, 1, 1);
-    render2dbox(c, xsz, 0, h);
-    render2dbox(c, 0, ysz, h);
+    gle::colorf(1, 1, 1);
+    gle::defvertex();
+    render2dbox(c, xsz, 0, h, m);
+    render2dbox(c, 0, ysz, h, m);
     c.add(vec(xsz, ysz, 0));
-    render2dbox(c, -xsz, 0, h);
-    render2dbox(c, 0, -ysz, h);
-    xtraverts += 16;
+    render2dbox(c, -xsz, 0, h, m);
+    render2dbox(c, 0, -ysz, h, m);
 }
 
 void renderellipse(vec &o, float xradius, float yradius, float yaw)
 {
-    glColor3f(0.5f, 0.5f, 0.5f);
-    glBegin(GL_LINE_LOOP);
+    gle::colorf(0.5f, 0.5f, 0.5f);
+    gle::defvertex();
+    gle::begin(GL_LINE_LOOP);
     loopi(15)
     {
         const vec2 &sc = sincos360[i*(360/15)];
-        vec p(xradius*sc.x, yradius*sc.y, 0);
-        p.rotate_around_z((yaw+90)*RAD);
-        p.add(o);
-        glVertex3fv(p.v);
+        gle::attrib(vec(xradius*sc.x, yradius*sc.y, 0).rotate_around_z((yaw+90)*RAD).add(o));
     }
-    glEnd();
+    xtraverts += gle::end();
 }
 
 struct batchedmodel
@@ -604,7 +601,7 @@ void renderbatchedmodel(model *m, batchedmodel &b)
     if(shadowmapping)
     {
         anim |= ANIM_NOSKIN;
-        if(renderpath!=R_FIXEDFUNCTION) setenvparamf("shadowintensity", SHPARAM_VERTEX, 1, b.transparent);
+        GLOBALPARAMF(shadowintensity, b.transparent);
     }
     else
     {
@@ -742,7 +739,12 @@ void endmodelquery()
 
 VAR(0, maxmodelradiusdistance, 10, 200, 1000);
 
-void rendermodelquery(model *m, dynent *d, const vec &center, float radius)
+static inline void enablecullmodelquery()
+{
+    startbb();
+}
+
+static inline void rendercullmodelquery(model *m, dynent *d, const vec &center, float radius)
 {
     if(fabs(camera1->o.x-center.x) < radius+1 &&
        fabs(camera1->o.y-center.y) < radius+1 &&
@@ -753,25 +755,67 @@ void rendermodelquery(model *m, dynent *d, const vec &center, float radius)
     }
     d->query = newquery(d);
     if(!d->query) return;
-    nocolorshader->set();
-    glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
-    glDepthMask(GL_FALSE);
     startquery(d->query);
     int br = int(radius*2)+1;
     drawbb(ivec(int(center.x-radius), int(center.y-radius), int(center.z-radius)), ivec(br, br, br));
     endquery(d->query);
-    glColorMask(COLORMASK, fading ? GL_FALSE : GL_TRUE);
-    glDepthMask(GL_TRUE);
 }
 
-extern int oqfrags;
+static inline void disablecullmodelquery()
+{
+    endbb();
+}
+
+static inline int cullmodel(model *m, const vec &center, float radius, int flags, dynent *d = NULL, bool shadow = false)
+{
+    if(flags&MDL_CULL_DIST && center.dist(camera1->o)/radius>maxmodelradiusdistance) return MDL_CULL_DIST;
+    if(flags&MDL_CULL_VFC)
+    {
+        if(reflecting || refracting)
+        {
+            if(reflecting || refracting>0)
+            {
+                if(center.z+radius<=reflectz) return MDL_CULL_VFC;
+            }
+            else
+            {
+                if(fogging && center.z+radius<reflectz-refractfog) return MDL_CULL_VFC;
+                if(!shadow && center.z-radius>=reflectz) return MDL_CULL_VFC;
+            }
+            if(center.dist(camera1->o)-radius>reflectdist) return MDL_CULL_VFC;
+        }
+        if(isfoggedsphere(radius, center)) return MDL_CULL_VFC;
+        if(shadowmapping && !isshadowmapcaster(center, radius)) return MDL_CULL_VFC;
+    }
+    if(shadowmapping)
+    {
+        if(d)
+        {
+            if(flags&MDL_CULL_OCCLUDED && d->occluded>=OCCLUDE_PARENT) return MDL_CULL_OCCLUDED;
+            if(flags&MDL_CULL_QUERY && d->occluded+1>=OCCLUDE_BB && d->query && d->query->owner==d && checkquery(d->query)) return MDL_CULL_QUERY;
+        }
+        if(!addshadowmapcaster(center, radius, radius)) return MDL_CULL_VFC;
+    }
+    else if(flags&MDL_CULL_OCCLUDED && modeloccluded(center, radius))
+    {
+        if(!reflecting && !refracting && d) d->occluded = OCCLUDE_PARENT;
+        return MDL_CULL_OCCLUDED;
+    }
+    else if(flags&MDL_CULL_QUERY && d->query && d->query->owner==d && checkquery(d->query))
+    {
+        if(!reflecting && !refracting && d->occluded<OCCLUDE_BB) d->occluded++;
+        return MDL_CULL_QUERY;
+    }
+    return 0;
+}
 
 void renderradius(const vec &o, float radius)
 {
-    glColor3f(0.5f, 0.5f, 0.5f);
+    gle::colorf(0.5f, 0.5f, 0.5f);
+    gle::defvertex();
     loopk(3)
     {
-        glBegin(GL_LINE_LOOP);
+        gle::begin(GL_LINE_LOOP);
         loopi(16)
         {
             vec d = o;
@@ -787,9 +831,9 @@ void renderradius(const vec &o, float radius)
                     d.add(vec(radius*cosf(2*M_PI*i/16.0f), radius*sinf(2*M_PI*i/16.0f), 0).rotate_around_z(90*RAD));
                     break;
             }
-            glVertex3fv(d.v);
+            gle::attrib(d);
         }
-        glEnd();
+        xtraverts += gle::end();
     }
 }
 
@@ -800,10 +844,15 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
     if(!m) return;
     vec center(0, 0, 0), bbradius(0, 0, 0);
     float radius = 0;
-    bool shadow = !shadowmap && !glaring && (flags&(MDL_SHADOW|MDL_DYNSHADOW)) && showblobs,
-         doOQ = flags&MDL_CULL_QUERY && hasOQ && oqfrags && oqdynent;
+    bool shadow = !shadowmap && !glaring && (flags&(MDL_SHADOW|MDL_DYNSHADOW)) && showblobs;
+
     if(flags&(MDL_CULL_VFC|MDL_CULL_DIST|MDL_CULL_OCCLUDED|MDL_CULL_QUERY|MDL_SHADOW|MDL_DYNSHADOW))
     {
+        if(flags&MDL_CULL_QUERY)
+        {
+            if(!oqfrags || !oqdynent || !d) flags &= ~MDL_CULL_QUERY;
+        }
+
         m->boundbox(center, bbradius);
         radius = bbradius.magnitude();
         if(d && d->ragdoll)
@@ -819,60 +868,26 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
             center.add(o);
         }
         radius *= size;
-        if(flags&MDL_CULL_DIST && center.dist(camera1->o)/radius>maxmodelradiusdistance) return;
-        if(flags&MDL_CULL_VFC)
+
+        int culled = cullmodel(m, center, radius, flags, d, shadow);
+        if(culled)
         {
-            if(reflecting || refracting)
+            if(culled&(MDL_CULL_OCCLUDED|MDL_CULL_QUERY) && flags&MDL_CULL_QUERY && !reflecting && !refracting)
             {
-                if(reflecting || refracting>0)
-                {
-                    if(center.z+radius<=reflectz) return;
-                }
-                else
-                {
-                    if(fogging && center.z+radius<reflectz-refractfog) return;
-                    if(!shadow && center.z-radius>=reflectz) return;
-                }
-                if(center.dist(camera1->o)-radius>reflectdist) return;
-            }
-            if(isfoggedsphere(radius, center)) return;
-            if(shadowmapping && !isshadowmapcaster(center, radius)) return;
-        }
-        if(shadowmapping)
-        {
-            if(d)
-            {
-                if(flags&MDL_CULL_OCCLUDED && d->occluded>=OCCLUDE_PARENT) return;
-                if(doOQ && d->occluded+1>=OCCLUDE_BB && d->query && d->query->owner==d && checkquery(d->query)) return;
-            }
-            if(!addshadowmapcaster(center, radius, radius)) return;
-        }
-        else if(flags&MDL_CULL_OCCLUDED && modeloccluded(center, radius))
-        {
-            if(!reflecting && !refracting && d)
-            {
-                d->occluded = OCCLUDE_PARENT;
-                if(doOQ) rendermodelquery(m, d, center, radius);
-            }
-            return;
-        }
-        else if(doOQ && d && d->query && d->query->owner==d && checkquery(d->query))
-        {
-            if(!reflecting && !refracting)
-            {
-                if(d->occluded<OCCLUDE_BB) d->occluded++;
-                rendermodelquery(m, d, center, radius);
+                enablecullmodelquery();
+                rendercullmodelquery(m, d, center, radius);
+                disablecullmodelquery();
             }
             return;
         }
+
+        if(reflecting || refracting || shadowmapping) flags &= ~MDL_CULL_QUERY;
     }
 
     if(flags&MDL_NORENDER) anim |= ANIM_NORENDER;
     else if(showboundingbox && !shadowmapping && !reflecting && !refracting)
     {
-        glPushMatrix();
-        lineshader->set();
-        glDisable(GL_TEXTURE_2D);
+        notextureshader->set();
         glDisable(GL_CULL_FACE);
         glEnable(GL_BLEND);
         glBlendFunc(GL_ONE, GL_ONE);
@@ -882,13 +897,14 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
             if(d->ragdoll && (d->state == CS_DEAD || d->state == CS_WAITING))
             {
                 loopv(d->ragdoll->skel->verts) renderradius(d->ragdoll->verts[i].pos, d->ragdoll->skel->verts[i].radius);
+                gle::colorf(0.5f, 1, 1);
+                gle::defvertex();
                 loopv(d->ragdoll->skel->tris)
                 {
                     ragdollskel::tri &t = d->ragdoll->skel->tris[i];
-                    glColor3f(0.5f, 1, 1);
-                    glBegin(GL_LINE_LOOP);
-                    loopk(3) glVertex3fv(d->ragdoll->verts[t.vert[k]].pos.v);
-                    glEnd();
+                    gle::begin(GL_LINE_LOOP);
+                    loopk(3) gle::attrib(d->ragdoll->verts[t.vert[k]].pos);
+                    gle::end();
                 }
             }
             else physics::complexboundbox(d);
@@ -900,18 +916,17 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
             else m->boundbox(center, radius);
             center.mul(size);
             radius.mul(size);
-            glTranslatef(o.x, o.y, o.z);
-            glRotatef(yaw, 0, 0, 1);
-            glRotatef(-roll, 1, 0, 0);
-            glRotatef(-pitch, 0, 1, 0);
-            render3dbox(center, radius.z, radius.z, radius.x, radius.y);
+            matrix4x3 m;
+            m.identity();
+            m.settranslation(o);
+            m.rotate_around_z(yaw*RAD);
+            m.rotate_around_x(roll*-RAD);
+            m.rotate_around_y(pitch*-RAD);
+            render3dbox(center, radius.z, radius.z, radius.x, radius.y, &m);
         }
 
-        defaultshader->set();
-        glEnable(GL_TEXTURE_2D);
         glEnable(GL_CULL_FACE);
         glDisable(GL_BLEND);
-        glPopMatrix();
     }
 
     vec lightcolor(1, 1, 1), lightdir(0, 0, 1);
@@ -950,9 +965,14 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
                 light->millis = lastmillis;
             }
         }
-        if(light) { lightcolor = light->color; lightdir = light->dir; lightmaterial = light->material; }
+        if(light)
+        {
+            lightcolor = light->color;
+            lightdir = light->dir;
+            lightmaterial = light->material;
+            if(flags&MDL_LIGHTFX) lightcolor.max(light->effect).lerp(light->effect, 0.6f*max(max(light->effect.x, light->effect.y), light->effect.z));
+        }
         if(flags&MDL_DYNLIGHT) dynlightreaching(pos, lightcolor, lightdir);
-        if(flags&MDL_LIGHTFX) lightcolor.max(light->effect).lerp(light->effect, 0.6f*max(max(light->effect.x, light->effect.y), light->effect.z));
     }
 
     if(a) for(int i = 0; a[i].tag; i++)
@@ -961,8 +981,6 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
         //if(a[i].m && a[i].m->type()!=m->type()) a[i].m = NULL;
     }
 
-    if(!d || reflecting || refracting || shadowmapping) doOQ = false;
-
     if(numbatches>=0)
     {
         modelbatch &mb = addbatchedmodel(m);
@@ -990,7 +1008,7 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
         b.d = d;
         b.attached = a ? modelattached.length() : -1;
         if(a) for(int i = 0;; i++) { modelattached.add(a[i]); if(!a[i].tag) break; }
-        if(doOQ) d->query = b.query = newquery(d);
+        if(flags&MDL_CULL_QUERY) d->query = b.query = newquery(d);
         return;
     }
 
@@ -1006,14 +1024,14 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
     if(shadowmapping)
     {
         anim |= ANIM_NOSKIN;
-        if(renderpath!=R_FIXEDFUNCTION) setenvparamf("shadowintensity", SHPARAM_VERTEX, 1, trans);
+        GLOBALPARAMF(shadowintensity, trans);
     }
     else
     {
         if(flags&MDL_FULLBRIGHT) anim |= ANIM_FULLBRIGHT;
     }
 
-    if(doOQ)
+    if(flags&MDL_CULL_QUERY)
     {
         d->query = newquery(d);
         if(d->query) startquery(d->query);
@@ -1021,7 +1039,7 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
 
     m->render(anim, basetime, basetime2, o, yaw, pitch, roll, d, a, lightcolor, lightdir, lightmaterial, trans, size);
 
-    if(doOQ && d->query) endquery(d->query);
+    if(flags&MDL_CULL_QUERY && d->query) endquery(d->query);
 
     m->endrender();
 }
@@ -1066,7 +1084,7 @@ ICOMMAND(0, findanims, "s", (char *name),
     string num;
     loopv(anims)
     {
-        formatstring(num)("%d", anims[i]);
+        formatstring(num, "%d", anims[i]);
         if(i > 0) buf.add(' ');
         buf.put(num, strlen(num));
     }
@@ -1077,14 +1095,14 @@ ICOMMAND(0, findanims, "s", (char *name),
 void loadskin(const char *dir, const char *altdir, Texture *&skin, Texture *&masks) // model skin sharing
 {
     string dirs[3];
-    formatstring(dirs[0])("models/%s/", dir);
-    formatstring(dirs[1])("models/%s/", altdir);
-    formatstring(dirs[2])("textures/");
+    formatstring(dirs[0], "%s/", dir);
+    formatstring(dirs[1], "%s/", altdir);
+    formatstring(dirs[2], "textures/");
     masks = notexture;
 
     #define tryload(tex, prefix, cmd, path) loopi(4) { if((tex = textureload(makerelpath(i < 3 ? dirs[i] : "", path, prefix, cmd), 0, true, false)) != notexture) break; }
     tryload(skin, NULL, NULL, "skin");
-    tryload(masks, "<stub>", NULL, "masks");
+    tryload(masks, NULL, NULL, "masks");
 }
 
 void setbbfrommodel(dynent *d, const char *mdl, float size)
