@@ -3,7 +3,7 @@ extern int rtsharefb, rtscissor, blurtile;
 struct rendertarget
 {
     int texw, texh, vieww, viewh;
-    GLenum colorfmt, depthfmt, target;
+    GLenum colorfmt, depthfmt;
     GLuint rendertex, renderfb, renderdb, blurtex, blurfb, blurdb;
     int blursize;
     float blursigma;
@@ -16,7 +16,7 @@ struct rendertarget
 
     bool initialized;
 
-    rendertarget() : texw(0), texh(0), vieww(0), viewh(0), colorfmt(GL_FALSE), depthfmt(GL_FALSE), target(GL_TEXTURE_2D), rendertex(0), renderfb(0), renderdb(0), blurtex(0), blurfb(0), blurdb(0), blursize(0), blursigma(0), initialized(false)
+    rendertarget() : texw(0), texh(0), vieww(0), viewh(0), colorfmt(GL_FALSE), depthfmt(GL_FALSE), rendertex(0), renderfb(0), renderdb(0), blurtex(0), blurfb(0), blurdb(0), blursize(0), blursigma(0), initialized(false)
     {
     }
 
@@ -24,7 +24,7 @@ struct rendertarget
 
     virtual GLenum attachment() const
     {
-        return GL_COLOR_ATTACHMENT0_EXT;
+        return GL_COLOR_ATTACHMENT0;
     }
 
     virtual const GLenum *colorformats() const
@@ -63,40 +63,32 @@ struct rendertarget
 
     void setupblur()
     {
-        if(!hasFBO) return;
         if(!blurtex) glGenTextures(1, &blurtex);
-        createtexture(blurtex, texw, texh, NULL, 3, 1, colorfmt, target);
+        createtexture(blurtex, texw, texh, NULL, 3, 1, colorfmt);
 
         if(!swaptexs() || rtsharefb) return;
         if(!blurfb) glGenFramebuffers_(1, &blurfb);
-        glBindFramebuffer_(GL_FRAMEBUFFER_EXT, blurfb);
-        glFramebufferTexture2D_(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, target, blurtex, 0);
+        glBindFramebuffer_(GL_FRAMEBUFFER, blurfb);
+        glFramebufferTexture2D_(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, blurtex, 0);
         if(depthtest())
         {
             if(!blurdb) glGenRenderbuffers_(1, &blurdb);
             glGenRenderbuffers_(1, &blurdb);
-            glBindRenderbuffer_(GL_RENDERBUFFER_EXT, blurdb);
-            glRenderbufferStorage_(GL_RENDERBUFFER_EXT, depthfmt, texw, texh);
-            glFramebufferRenderbuffer_(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, GL_RENDERBUFFER_EXT, blurdb);
+            glBindRenderbuffer_(GL_RENDERBUFFER, blurdb);
+            glRenderbufferStorage_(GL_RENDERBUFFER, depthfmt, texw, texh);
+            glFramebufferRenderbuffer_(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, blurdb);
         }
-        glBindFramebuffer_(GL_FRAMEBUFFER_EXT, 0);
+        glBindFramebuffer_(GL_FRAMEBUFFER, 0);
     }
 
-    virtual bool shadowcompare() const { return false; }
-
     void setup(int w, int h)
     {
-        if(hasFBO)
-        {
-            if(!renderfb) glGenFramebuffers_(1, &renderfb);
-            glBindFramebuffer_(GL_FRAMEBUFFER_EXT, renderfb);
-        }
+        if(!renderfb) glGenFramebuffers_(1, &renderfb);
+        glBindFramebuffer_(GL_FRAMEBUFFER, renderfb);
         if(!rendertex) glGenTextures(1, &rendertex);
 
-        target = texrect() ? GL_TEXTURE_RECTANGLE_ARB : GL_TEXTURE_2D;
-
         GLenum attach = attachment();
-        if(hasFBO && attach == GL_DEPTH_ATTACHMENT_EXT)
+        if(attach == GL_DEPTH_ATTACHMENT)
         {
             glDrawBuffer(GL_NONE);
             glReadBuffer(GL_NONE);
@@ -106,49 +98,30 @@ struct rendertarget
         int find = 0;
         do
         {
-            createtexture(rendertex, w, h, NULL, 3, filter() ? 1 : 0, colorfmt ? colorfmt : colorfmts[find], target);
-            if(!hasFBO) break;
-            else
-            {
-                glFramebufferTexture2D_(GL_FRAMEBUFFER_EXT, attach, target, rendertex, 0);
-                if(glCheckFramebufferStatus_(GL_FRAMEBUFFER_EXT)==GL_FRAMEBUFFER_COMPLETE_EXT) break;
-            }
+            createtexture(rendertex, w, h, NULL, 3, filter() ? 1 : 0, colorfmt ? colorfmt : colorfmts[find]);
+            glFramebufferTexture2D_(GL_FRAMEBUFFER, attach, GL_TEXTURE_2D, rendertex, 0);
+            if(glCheckFramebufferStatus_(GL_FRAMEBUFFER)==GL_FRAMEBUFFER_COMPLETE) break;
         }
         while(!colorfmt && colorfmts[++find]);
         if(!colorfmt) colorfmt = colorfmts[find];
 
-        if(attach == GL_DEPTH_ATTACHMENT_EXT && shadowcompare())
-        {
-            if(hasDT && hasSH)
-            {
-                glTexParameteri(target, GL_TEXTURE_COMPARE_MODE_ARB, GL_COMPARE_R_TO_TEXTURE_ARB);
-                glTexParameteri(target, GL_TEXTURE_COMPARE_FUNC_ARB, GL_GEQUAL);
-                glTexParameteri(target, GL_DEPTH_TEXTURE_MODE_ARB, GL_LUMINANCE);
-            }
-            else
-            {
-                glTexParameteri(target, GL_TEXTURE_COMPARE_SGIX, GL_TRUE);
-                glTexParameteri(target, GL_TEXTURE_COMPARE_OPERATOR_SGIX, GL_TEXTURE_GEQUAL_R_SGIX);
-            }
-        }
-
-        if(hasFBO && attach != GL_DEPTH_ATTACHMENT_EXT && depthtest())
+        if(attach != GL_DEPTH_ATTACHMENT && depthtest())
         {
             if(!renderdb) { glGenRenderbuffers_(1, &renderdb); depthfmt = GL_FALSE; }
-            if(!depthfmt) glBindRenderbuffer_(GL_RENDERBUFFER_EXT, renderdb);
+            if(!depthfmt) glBindRenderbuffer_(GL_RENDERBUFFER, renderdb);
             const GLenum *depthfmts = depthformats();
             find = 0;
             do
             {
-                if(!depthfmt) glRenderbufferStorage_(GL_RENDERBUFFER_EXT, depthfmts[find], w, h);
-                glFramebufferRenderbuffer_(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, GL_RENDERBUFFER_EXT, renderdb);
-                if(glCheckFramebufferStatus_(GL_FRAMEBUFFER_EXT)==GL_FRAMEBUFFER_COMPLETE_EXT) break;
+                if(!depthfmt) glRenderbufferStorage_(GL_RENDERBUFFER, depthfmts[find], w, h);
+                glFramebufferRenderbuffer_(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, renderdb);
+                if(glCheckFramebufferStatus_(GL_FRAMEBUFFER)==GL_FRAMEBUFFER_COMPLETE) break;
             }
             while(!depthfmt && depthfmts[++find]);
             if(!depthfmt) depthfmt = depthfmts[find];
         }
  
-        if(hasFBO) glBindFramebuffer_(GL_FRAMEBUFFER_EXT, 0);
+        glBindFramebuffer_(GL_FRAMEBUFFER, 0);
 
         texw = w;
         texh = h;
@@ -195,20 +168,17 @@ struct rendertarget
         return false;
     }
 
-    virtual void rendertiles()
+    void rendertiles()
     {
-        glBegin(GL_QUADS);
-        float wscale = vieww, hscale = viewh;
-        if(target!=GL_TEXTURE_RECTANGLE_ARB)
-        {
-            wscale /= texw;
-            hscale /= texh;
-        }
+        float wscale = vieww/float(texw), hscale = viewh/float(texh);
         if(blurtile && scissorx1 < scissorx2 && scissory1 < scissory2)
         {
             uint tiles[sizeof(blurtiles)/sizeof(uint)];
             memcpy(tiles, blurtiles, sizeof(blurtiles));
 
+            LOCALPARAMF(screentexcoord0, wscale*0.5f, hscale*0.5f, wscale*0.5f, hscale*0.5f);
+            gle::defvertex(2);
+            gle::begin(GL_QUADS);
             float tsz = 1.0f/BLURTILES;
             loop(y, BLURTILES+1)
             {
@@ -228,35 +198,23 @@ struct rendertarget
                           tw = (x-xstart)*tsz,
                           th = (yend-y)*tsz,
                           vx = 2*tx - 1, vy = 2*ty - 1, vw = tw*2, vh = th*2;
-                    tx *= wscale;
-                    ty *= hscale;
-                    tw *= wscale;
-                    th *= hscale;
-                    glTexCoord2f(tx,    ty);    glVertex2f(vx,    vy);
-                    glTexCoord2f(tx+tw, ty);    glVertex2f(vx+vw, vy);
-                    glTexCoord2f(tx+tw, ty+th); glVertex2f(vx+vw, vy+vh);
-                    glTexCoord2f(tx,    ty+th); glVertex2f(vx,    vy+vh);
+                    gle::attribf(vx,    vy);
+                    gle::attribf(vx+vw, vy);
+                    gle::attribf(vx+vw, vy+vh);
+                    gle::attribf(vx,    vy+vh);
                 }
             }
+            gle::end();
         }
         else
         {
-            glTexCoord2f(0,      0);      glVertex2f(-1, -1);
-            glTexCoord2f(wscale, 0);      glVertex2f( 1, -1);
-            glTexCoord2f(wscale, hscale); glVertex2f( 1,  1);
-            glTexCoord2f(0,      hscale); glVertex2f(-1,  1);
+            screenquad(wscale, hscale);
         }
-        glEnd();
     }
 
     void blur(int wantsblursize, float wantsblursigma, int x, int y, int w, int h, bool scissor)
     {
-        if(!hasFBO)
-        {
-            x += screen->w-vieww;
-            y += screen->h-viewh;
-        }
-        else if(!blurtex) setupblur();
+        if(!blurtex) setupblur();
         if(blursize!=wantsblursize || (wantsblursize && blursigma!=wantsblursigma))
         {
             setupblurkernel(wantsblursize, wantsblursigma, blurweights, bluroffsets);
@@ -275,18 +233,13 @@ struct rendertarget
 
         loopi(2)
         {
-            setblurshader(i, target == GL_TEXTURE_RECTANGLE_ARB ? 1 : (i ? texh : texw), blursize, blurweights, bluroffsets, target);
+            setblurshader(i, i ? texh : texw, blursize, blurweights, bluroffsets);
 
-            if(hasFBO)
-            {
-                if(!swaptexs() || rtsharefb) glFramebufferTexture2D_(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, target, i ? rendertex : blurtex, 0);
-                else glBindFramebuffer_(GL_FRAMEBUFFER_EXT, i ? renderfb : blurfb);
-                glBindTexture(target, i ? blurtex : rendertex);
-            }
+            if(!swaptexs() || rtsharefb) glFramebufferTexture2D_(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, i ? rendertex : blurtex, 0);
+            else glBindFramebuffer_(GL_FRAMEBUFFER, i ? renderfb : blurfb);
+            glBindTexture(GL_TEXTURE_2D, i ? blurtex : rendertex);
 
             rendertiles();
-
-            if(!hasFBO) glCopyTexSubImage2D(target, 0, x-(screen->w-vieww), y-(screen->h-viewh), x, y, w, h);
         }
 
         if(scissor) glDisable(GL_SCISSOR_TEST);
@@ -351,39 +304,21 @@ struct rendertarget
 
     virtual void doclear() {}
 
-    virtual bool screenview() const { return false; }
-    virtual bool texrect() const { return false; }
+    virtual bool screenrect() const { return false; }
     virtual bool filter() const { return true; }
 
     void render(int w, int h, int blursize = 0, float blursigma = 0)
     {
         w = min(w, hwtexsize);
         h = min(h, hwtexsize);
-        if(texrect())
+        if(screenrect())
         {
-            if(w > screen->w) w = screen->w;
-            if(h > screen->h) h = screen->h;
-            vieww = w;
-            viewh = h;
+            if(w > screenw) w = screenw;
+            if(h > screenh) h = screenh;
         }
-        else if(screenview())
-        {
-            while(screen->w < (w*3)/4) w /= 2;
-            while(screen->h < (h*3)/4) h /= 2;
-            vieww = min(w, screen->w);
-            viewh = min(h, screen->h);
-        }
-        else 
-        {
-            if(!hasFBO)
-            {
-                while(w > screen->w) w /= 2;
-                while(h > screen->h) h /= 2;
-            }
-            vieww = w;
-            viewh = h;
-        }
-        if(w!=texw || h!=texh || (texrect() ? target!=GL_TEXTURE_RECTANGLE_ARB : target!=GL_TEXTURE_2D) || (hasFBO && (swaptexs() && !rtsharefb ? !blurfb : blurfb))) cleanup();
+        vieww = w;
+        viewh = h;
+        if(w!=texw || h!=texh || (swaptexs() && !rtsharefb ? !blurfb : blurfb)) cleanup();
         
         if(!filter())
         {
@@ -399,24 +334,20 @@ struct rendertarget
  
         if(!shouldrender()) return;
 
-        if(hasFBO)
+        if(blursize && !blurtex) setupblur();
+        if(swaptexs() && blursize)
         {
-            if(blursize && !blurtex) setupblur();
-            if(swaptexs() && blursize)
+            swap(rendertex, blurtex);
+            if(!rtsharefb)
             {
-                swap(rendertex, blurtex);
-                if(!rtsharefb)
-                {
-                    swap(renderfb, blurfb);
-                    swap(renderdb, blurdb);
-                }
+                swap(renderfb, blurfb);
+                swap(renderdb, blurdb);
             }
-            glBindFramebuffer_(GL_FRAMEBUFFER_EXT, renderfb);
-            if(swaptexs() && blursize && rtsharefb)
-                glFramebufferTexture2D_(GL_FRAMEBUFFER_EXT, attachment(), target, rendertex, 0);
-            glViewport(0, 0, vieww, viewh);
         }
-        else glViewport(screen->w-vieww, screen->h-viewh, vieww, viewh);
+        glBindFramebuffer_(GL_FRAMEBUFFER, renderfb);
+        if(swaptexs() && blursize && rtsharefb)
+            glFramebufferTexture2D_(GL_FRAMEBUFFER, attachment(), GL_TEXTURE_2D, rendertex, 0);
+        glViewport(0, 0, vieww, viewh);
 
         doclear();
 
@@ -424,18 +355,12 @@ struct rendertarget
         bool scissoring = rtscissor && scissorrender(sx, sy, sw, sh) && sw > 0 && sh > 0;
         if(scissoring)
         {
-            if(!hasFBO)
-            {
-                sx += screen->w-vieww;
-                sy += screen->h-viewh;
-            }
             glScissor(sx, sy, sw, sh);
             glEnable(GL_SCISSOR_TEST);
         }
         else
         {
-            sx = hasFBO ? 0 : screen->w-vieww;
-            sy = hasFBO ? 0 : screen->h-viewh;
+            sx = sy = 0;
             sw = vieww;
             sh = viewh;
         }
@@ -452,18 +377,6 @@ struct rendertarget
 
         if(succeeded)
         {
-            if(!hasFBO)
-            {
-                glBindTexture(target, rendertex);
-                if(!initialized) 
-                {
-                    sx = screen->w-vieww;
-                    sy = screen->h-viewh;
-                    sw = vieww;
-                    sh = viewh;
-                }
-                glCopyTexSubImage2D(target, 0, sx-(screen->w-vieww), sy-(screen->h-viewh), sx, sy, sw, sh);
-            }
             initialized = true;
 
             if(blursize) doblur(blursize, blursigma);
@@ -471,8 +384,8 @@ struct rendertarget
 
         RESTORECOLORMASK
 
-        if(hasFBO) glBindFramebuffer_(GL_FRAMEBUFFER_EXT, 0);
-        glViewport(0, 0, screen->w, screen->h);
+        glBindFramebuffer_(GL_FRAMEBUFFER, 0);
+        glViewport(0, 0, screenw, screenh);
     }
 
     virtual void dodebug(int w, int h) {}
@@ -486,19 +399,21 @@ struct rendertarget
             sw = int(0.5f*(scissorx2 - scissorx1)*w),
             sh = int(0.5f*(scissory2 - scissory1)*h);
         if(flipdebug()) { sy = h - sy; sh = -sh; }
-        glBegin(lines ? GL_LINE_LOOP : GL_TRIANGLE_STRIP);
-        glVertex2i(sx,      sy);
-        glVertex2i(sx + sw, sy);
-        if(lines) glVertex2i(sx + sw, sy + sh);
-        glVertex2i(sx,      sy + sh);
-        if(!lines) glVertex2i(sx + sw, sy + sh);
-        glEnd();
+        gle::defvertex(2);
+        gle::begin(lines ? GL_LINE_LOOP : GL_TRIANGLE_STRIP);
+        gle::attribf(sx,      sy);
+        gle::attribf(sx + sw, sy);
+        if(lines) gle::attribf(sx + sw, sy + sh);
+        gle::attribf(sx,      sy + sh);
+        if(!lines) gle::attribf(sx + sw, sy + sh);
+        gle::end();
     }
 
     void debugblurtiles(int w, int h, bool lines = false)
     {
         if(!blurtile) return;
         float vxsz = float(w)/BLURTILES, vysz = float(h)/BLURTILES;
+        gle::defvertex(2);
         loop(y, BLURTILES+1)
         {
             uint mask = blurtiles[y];
@@ -519,14 +434,14 @@ struct rendertarget
                 if(flipdebug()) { vy = h - vy; vh = -vh; }
                 loopi(lines ? 1 : 2)
                 {
-                    if(!lines) glColor3f(1, 1, i ? 1.0f : 0.5f);
-                    glBegin(lines || i ? GL_LINE_LOOP : GL_TRIANGLE_STRIP);
-                    glVertex2f(vx,    vy);
-                    glVertex2f(vx+vw, vy);
-                    if(lines || i) glVertex2f(vx+vw, vy+vh);
-                    glVertex2f(vx,    vy+vh);
-                    if(!lines && !i) glVertex2f(vx+vw, vy+vh);
-                    glEnd();
+                    if(!lines) gle::colorf(1, 1, i ? 1.0f : 0.5f);
+                    gle::begin(lines || i ? GL_LINE_LOOP : GL_TRIANGLE_STRIP);
+                    gle::attribf(vx,    vy);
+                    gle::attribf(vx+vw, vy);
+                    if(lines || i) gle::attribf(vx+vw, vy+vh);
+                    gle::attribf(vx,    vy+vh);
+                    if(!lines && !i) gle::attribf(vx+vw, vy+vh);
+                    gle::end();
                 }
             }
         }
@@ -535,26 +450,14 @@ struct rendertarget
     void debug()
     {
         if(!rendertex) return;
-        int w = min(screen->w, screen->h)/2, h = (w*screen->h)/screen->w;
-        (target==GL_TEXTURE_RECTANGLE_ARB ? rectshader : defaultshader)->set();
-        glColor3f(1, 1, 1);
-        glEnable(target);
-        glBindTexture(target, rendertex);
-        float tx1 = 0, tx2 = vieww, ty1 = 0, ty2 = viewh;
-        if(target!=GL_TEXTURE_RECTANGLE_ARB)
-        {
-            tx2 /= vieww;
-            ty2 /= viewh;
-        }
+        int w = min(screenw, screenh)/2, h = (w*screenh)/screenw;
+        hudshader->set();
+        gle::colorf(1, 1, 1);
+        glBindTexture(GL_TEXTURE_2D, rendertex);
+        float tx1 = 0, tx2 = 1, ty1 = 0, ty2 = 1;
         if(flipdebug()) swap(ty1, ty2);
-        glBegin(GL_TRIANGLE_STRIP);
-        glTexCoord2f(tx1, ty1); glVertex2i(0, 0);
-        glTexCoord2f(tx2, ty1); glVertex2i(w, 0);
-        glTexCoord2f(tx1, ty2); glVertex2i(0, h);
-        glTexCoord2f(tx2, ty2); glVertex2i(w, h);
-        glEnd();
-        notextureshader->set();
-        glDisable(target);
+        hudquad(0, 0, w, h, tx1, ty1, tx2-tx1, ty2-ty1);
+        hudnotextureshader->set();
         dodebug(w, h);
     }
 };
