@@ -42,7 +42,7 @@ SVAR(IDF_READONLY, versionplatlongname, plat_longname(CUR_PLATFORM));
 VAR(IDF_READONLY, versionplatform, 0, CUR_PLATFORM, VAR_MAX);
 VAR(IDF_READONLY, versionarch, 0, CUR_ARCH, VAR_MAX);
 VAR(IDF_READONLY, versioncrc, 0, 0, VAR_MAX);
-SVAR(IDF_READONLY, versionbranch, "none");
+SVARF(IDF_READONLY, versionbranch, "none", versionbranch[MAXBRANCHLEN+1] = 0);
 #ifdef STANDALONE
 VAR(IDF_READONLY, versionisserver, 0, 1, 1);
 #else
@@ -50,12 +50,9 @@ VAR(IDF_READONLY, versionisserver, 0, 0, 1);
 #endif
 ICOMMAND(0, platname, "ii", (int *p, int *g), result(*p >= 0 && *p < MAX_PLATFORMS ? (*g!=0 ? plat_longname(*p) : plat_name(*p)) : ""));
 
-SVAR(IDF_READONLY, systemuser, "none");
-SVAR(IDF_READONLY, systemhost, "unknown");
-
 VAR(0, rehashing, 1, 0, -1);
 
-const char * const disc_reasons[] = { "normal", "end of packet", "client num", "user was kicked", "message error", "address is banned", "server is in private mode", "server is password protected", "server requires pure official builds", "server is at maximum capacity", "server and client are incompatible", "connection timed out", "packet overflow", "hostname lookup failure", "server shutting down" };
+const char * const disc_reasons[] = { "normal", "end of packet", "client num", "user was kicked", "message error", "address is banned", "server is in private mode", "server is password protected", "server requires pure official builds", "server is at maximum capacity", "server and client are incompatible", "connection timed out", "packet overflow", "server shutting down", "hostname lookup failure", "client with same handle authenticated" };
 
 SVAR(IDF_PERSIST, logtimeformat, "%Y-%m-%d %H:%M.%S");
 VAR(IDF_PERSIST, logtimelocal, 0, 1, 1); // use clockoffset to localise
@@ -430,7 +427,6 @@ void process(ENetPacket *packet, int sender, int chan);
 
 int getservermtu() { return serverhost ? serverhost->mtu : -1; }
 void *getinfo(int i)            { return !clients.inrange(i) || clients[i]->type==ST_EMPTY ? NULL : clients[i]->info; }
-const char *gethostname(int i)  { int o = server::peerowner(i); return !clients.inrange(o) || clients[o]->type==ST_EMPTY ? "unknown" : clients[o]->hostname; }
 const char *gethostip(int i)    { int o = server::peerowner(i); return !clients.inrange(o) || clients[o]->type==ST_EMPTY ? "0.0.0.0" : clients[o]->hostip; }
 int getnumclients()             { return clients.length(); }
 uint getclientip(int n)         { int o = server::peerowner(n); return clients.inrange(o) && clients[o]->type==ST_TCPIP ? clients[o]->peer->address.host : 0; }
@@ -626,7 +622,6 @@ void localconnect(bool force)
         int cn = addclient(ST_LOCAL);
         clientdata &c = *clients[cn];
         c.peer = NULL;
-        copystring(c.hostname, "localhost");
         copystring(c.hostip, "127.0.0.1");
         conoutf("\fglocal client %d connected", c.num);
         client::gameconnect(false);
@@ -923,22 +918,6 @@ void serverslice(uint timeout)  // main server update, called from main loop in
                 c.peer->data = &c;
                 if(enet_address_get_host_ip(&c.peer->address, c.hostip, sizeof(c.hostip)) >= 0)
                 {
-                    if(enet_address_get_host(&c.peer->address, c.hostname, sizeof(c.hostname)) >= 0)
-                    {
-                        ENetAddress address;
-                        string ip;
-                        conoutf("checking reverse lookup of %s (%s)", c.hostip, c.hostname);
-                        if(enet_address_set_host(&address, c.hostname) < 0 || enet_address_get_host_ip(&address, ip, sizeof(ip)) < 0 || strcmp(ip, c.hostip))
-                        {
-                            conoutf("reverse lookup of %s (%s) failed (%s), using ip address", c.hostip, c.hostname, ip);
-                            copystring(c.hostname, c.hostip);
-                        }
-                    }
-                    else
-                    {
-                        conoutf("reverse lookup of %s failed, using ip address", c.hostip);
-                        copystring(c.hostname, c.hostip);
-                    }
                     int reason = server::clientconnect(c.num, c.peer->address.host);
                     if(reason) disconnect_client(c.num, reason);
                 }
@@ -1241,7 +1220,7 @@ static void setupwindow(const char *title)
     atexit(cleanupwindow);
 
     if(!setupsystemtray(WM_APP)) fatal("failed adding to system tray");
-    conoutf("identity: %s@%s v%s-%s%d-%s %s (%s) [0x%.8x]", systemuser, systemhost, versionstring, versionplatname, versionarch, versionbranch, versionisserver ? "server" : "client", versionrelease, versioncrc);
+    conoutf("version: %s-%s%d-%s %s (%s) [0x%.8x]", versionstring, versionplatname, versionarch, versionbranch, versionisserver ? "server" : "client", versionrelease, versioncrc);
 }
 
 static char *parsecommandline(const char *src, vector<char *> &args)
@@ -1437,7 +1416,7 @@ void setupserver()
 
 void initgame()
 {
-    conoutf("identity: %s@%s v%s-%s%d-%s %s (%s) [0x%.8x]", systemuser, systemhost, versionstring, versionplatname, versionarch, versionbranch, versionisserver ? "server" : "client", versionrelease, versioncrc);
+    conoutf("version: %s-%s%d-%s %s (%s) [0x%.8x]", versionstring, versionplatname, versionarch, versionbranch, versionisserver ? "server" : "client", versionrelease, versioncrc);
     server::start();
     loopv(gameargs)
     {
@@ -1699,35 +1678,9 @@ ICOMMAND(0, rehash, "i", (int *nosave), if(!(identflags&IDF_WORLD)) rehash(*nosa
 #endif
 void setverinfo(const char *bin)
 {
-    string buf;
     setvar("versioncrc", crcfile(bin));
     const char *vbranch = getenv(sup_var("BRANCH"));
-    setsvar("versionbranch", vbranch && *vbranch ? vbranch : "none");
-#ifdef WIN32
-    const char *suser = getenv("USERNAME");
-#else
-    const char *suser = getenv("USER");
-    if(!suser || !*suser)
-    { // only fall back to this if the variable isn't exported
-        passwd *p = getpwuid(geteuid());
-        if(p)
-        {
-            copystring(buf, p->pw_name);
-            suser = buf;
-        }
-    }
-#endif
-    setsvar("systemuser", suser && *suser ? suser : "none");
-#ifdef WIN32
-    const char *shost = getenv("COMPUTERNAME");
-#else
-    const char *shost = getenv("HOSTNAME");
-#endif
-    if(!shost || !*shost)
-    { // only fall back to this if the variable isn't exported
-        if(!gethostname(buf, sizeof(string)-1) && *buf) shost = buf;
-    }
-    setsvar("systemhost", shost && *shost ? shost : "unknown");
+    setsvar("versionbranch", vbranch);
 }
 
 volatile bool fatalsig = false;
