@@ -230,10 +230,10 @@ static serverinfo *newserver(const char *name, int port = SERVER_PORT, int prior
         delete si;
         return NULL;
     }
-    if(desc && *desc) copystring(si->sdesc, desc);
+    if(desc && *desc) copystring(si->sdesc, desc, MAXSDESCLEN+1);
     if(handle && *handle) copystring(si->authhandle, handle);
     if(flags && *flags) copystring(si->flags, flags);
-    if(branch && *branch) copystring(si->branch, branch);
+    if(branch && *branch) copystring(si->branch, branch, MAXBRANCHLEN+1);
 
     servers.add(si);
     sortedservers = false;
@@ -364,13 +364,13 @@ void checkpings()
         getstring(text, p);
         filterstring(si->map, text, false);
         getstring(text, p);
-        filterstring(si->sdesc, text);
+        filterstring(si->sdesc, text, true, true, true, false, MAXSDESCLEN+1);
         si->players.deletearrays();
         si->handles.deletearrays();
         if(gver >= 227)
         {
             getstring(text, p);
-            filterstring(si->branch, text);
+            filterstring(si->branch, text, true, true, true, false, MAXBRANCHLEN+1);
         }
         loopi(si->numplayers)
         {
