@@ -490,7 +490,7 @@ struct verinfo
         gpuglslver = getint(p);
         crc = getint(p);
         if(branch) delete[] branch;
-        getstring(text, p); branch = newstring(text);
+        getstring(text, p); branch = newstring(text, MAXBRANCHLEN);
         if(gpuvendor) delete[] gpuvendor;
         getstring(text, p); gpuvendor = newstring(text);
         if(gpurenderer) delete[] gpurenderer;
@@ -529,7 +529,7 @@ struct verinfo
         gpuglslver = v.gpuglslver;
         crc = v.crc;
         if(branch) delete[] branch;
-        branch = newstring(v.branch ? v.branch : "");
+        branch = newstring(v.branch ? v.branch : "", MAXBRANCHLEN);
         if(gpuvendor) delete[] gpuvendor;
         gpuvendor = newstring(v.gpuvendor ? v.gpuvendor : "");
         if(gpurenderer) delete[] gpurenderer;
@@ -655,7 +655,7 @@ struct clientstate
             weapwait[i] = weaptime[i] = weapload[i] = weapshot[i] = prevtime[0] = 0;
             if(full) ammo[i] = entid[i] = -1;
         }
-        lastweap.shrink(0);
+        if(full) lastweap.shrink(0);
     }
 
     void setweapstate(int weap, int state, int delay, int millis, int offtime = 0, bool blank = false)
@@ -857,40 +857,26 @@ struct clientstate
         }
         if(AA(actortype, maxcarry) && m_loadout(gamemode, mutators))
         {
-            int n = 0, musthave = AA(actortype, maxcarry);
-            loopj(W_LOADOUT) if(canrandweap(j + W_OFFSET)) musthave--;
-            vector<bool> forced;
-            loopj(W_LOADOUT) forced.add(false);
             vector<int> aweap;
-            loopj(W_LOADOUT) aweap.add(loadweap.inrange(j) ? loadweap[j] : 0);
-            loopvj(aweap)
+            loopj(AA(actortype, maxcarry)) aweap.add(loadweap.inrange(j) ? loadweap[j] : 0);
+            vector<int> rand, forcerand;
+            for(int t = W_OFFSET; t < W_ITEM; t++)
+                if(!hasweap(t, sweap) && m_check(W(t, modes), W(t, muts), gamemode, mutators) && !W(t, disabled) && aweap.find(t) < 0)
+                    (canrandweap(t) ? rand : forcerand).add(t);
+            loopj(AA(actortype, maxcarry))
             {
-                if(!aweap[j]) // specifically asking for random
+                if(aweap[j] <= 0) // specifically asking for random
                 {
-                    for(int t = rnd(W_ITEM-W_OFFSET)+W_OFFSET, r = 0; r < W_LOADOUT; r++)
+                    vector<int> &randsrc = rand.empty() ? forcerand : rand;
+                    if(!randsrc.empty())
                     {
-                        bool canuse = canrandweap(t);
-                        if(!canuse && musthave > 0 && !forced[t - W_OFFSET])
-                        {
-                            canuse = true;
-                            musthave--;
-                            forced[t - W_OFFSET] = true;
-                        }
-                        if(t >= W_OFFSET && t < W_ITEM && !hasweap(t, sweap) && m_check(W(t, modes), W(t, muts), gamemode, mutators) && !W(t, disabled) && canuse)
-                        {
-                            aweap[j] = t;
-                            break;
-                        }
-                        else if(++t >= W_ITEM) t = W_OFFSET;
+                        int i = rnd(randsrc.length());
+                        aweap[j] = randsrc.remove(i);
                     }
                 }
-                if(aweap[j] >= W_OFFSET && aweap[j] < W_ITEM && !hasweap(aweap[j], sweap) && m_check(W(aweap[j], modes), W(aweap[j], muts), gamemode, mutators) && !W(aweap[j], disabled))
-                {
-                    ammo[aweap[j]] = max(1, W(aweap[j], ammomax));
-                    if(!n) weapselect = aweap[j];
-                    if(++n >= AA(actortype, maxcarry)) break;
-                }
+                ammo[aweap[j]] = max(1, W(aweap[j], ammomax));
             }
+            weapselect = aweap[0];
         }
     }
 
@@ -1027,7 +1013,7 @@ struct gameent : dynent, clientstate
     float deltayaw, deltapitch, newyaw, newpitch, turnyaw, turnroll;
     vec head, torso, muzzle, origin, eject[2], waist, jet[3], legs, hrad, trad, lrad, toe[2];
     bool action[AC_MAX], conopen, k_up, k_down, k_left, k_right, obliterated, headless;
-    string hostname, hostip, name, handle, info, obit;
+    string hostip, name, handle, info, obit;
     vector<gameent *> dominating, dominated;
     vector<eventicon> icons;
     vector<stunevent> stuns;
@@ -1039,7 +1025,6 @@ struct gameent : dynent, clientstate
     {
         state = CS_DEAD;
         type = ENT_PLAYER;
-        copystring(hostname, "0.0.0.0");
         copystring(hostip, "0.0.0.0");
         name[0] = handle[0] = info[0] = obit[0] = '\0';
         removesounds();
@@ -1594,7 +1579,7 @@ struct cament
 
 namespace client
 {
-    extern int showpresence, showteamchange, needsmap, gettingmap;
+    extern int showpresence, showpresencehostinfo, showteamchange, needsmap, gettingmap;
     extern bool demoplayback, isready;
     extern vector<uchar> messages;
     extern void clearvotes(gameent *d, bool msg = false);
