@@ -1,19 +1,28 @@
-#!/bin/sh
+#! /bin/bash
+
 SEMABUILD_PWD=`pwd`
-SEMABUILD_BUILD="${HOME}/build"
+SEMABUILD_BUILD="${HOME}/deploy"
 SEMABUILD_DIR="${SEMABUILD_BUILD}/${BRANCH_NAME}"
-SEMABUILD_SCP='scp -BC -o StrictHostKeyChecking=no'
-SEMABUILD_TARGET='qreeves@icculus.org:/webspace/redeclipse.net/files'
 SEMABUILD_APT='DEBIAN_FRONTEND=noninteractive apt-get'
-SEMABUILD_SOURCE="http://redeclipse.net/files"
+SEMABUILD_DEST="https://${GITHUB_TOKEN}:x-oauth-basic@github.com/red-eclipse/deploy.git"
+SEMABUILD_SOURCE="https://raw.githubusercontent.com/red-eclipse/deploy/master"
+SEMABUILD_APPIMAGE="https://github.com/red-eclipse/appimage-builder.git"
+SEMABUILD_APPIMAGE_GH_DEST="red-eclipse/deploy"
 SEMABUILD_MODULES=`cat "${SEMABUILD_PWD}/.gitmodules" | grep '\[submodule "[^.]' | sed -e 's/^.submodule..//;s/..$//' | tr "\n" " " | sed -e 's/ $//'`
 SEMABUILD_ALLMODS="base ${SEMABUILD_MODULES}"
 SEMABUILD_DEPLOY="false"
 
 semabuild_setup() {
     echo "setting up ${BRANCH_NAME}..."
-    rm -rf "${SEMABUILD_DIR}"
-    rm -rf "${SEMABUILD_PWD}/data"
+    git config --global user.email "noreply@redeclipse.net" || return 1
+    git config --global user.name "Red Eclipse" || return 1
+    git config --global credential.helper store || return 1
+    echo "https://${GITHUB_TOKEN}:x-oauth-basic@github.com" > "${HOME}/.git-credentials"
+    rm -rf "${SEMABUILD_BUILD}" || return 1
+    rm -rf "${SEMABUILD_PWD}/data" || return 1
+    pushd "${HOME}" || return 1
+    git clone --depth 1 "${SEMABUILD_DEST}" || return 1
+    popd || return 1
     mkdir -pv "${SEMABUILD_DIR}" || return 1
     return 0
 }
@@ -42,7 +51,8 @@ semabuild_build() {
     make PLATFORM=crossmingw64 PLATFORM_BIN=amd64 INSTDIR=${SEMABUILD_DIR}/windows/bin/amd64 CFLAGS=-m64 CXXFLAGS=-m64 LDFLAGS=-m64 -C src clean install || return 1
     make PLATFORM=crossmingw32 PLATFORM_BIN=x86 INSTDIR=${SEMABUILD_DIR}/windows/bin/x86 CFLAGS=-m32 CXXFLAGS=-m32 LDFLAGS=-m32 -C src clean install || return 1
     make PLATFORM=linux64 PLATFORM_BIN=amd64 INSTDIR=${SEMABUILD_DIR}/linux/bin/amd64 CFLAGS=-m64 CXXFLAGS=-m64 LDFLAGS=-m64 -C src clean install || return 1
-    sudo apt-get -o Dpkg::Options::="--force-overwrite" -fy install gcc:i386 g++:i386 cpp:i386 g++-4.8:i386 gcc-4.8:i386 cpp-4.8:i386 binutils:i386 zlib1g-dev:i386 libsdl2-dev:i386 libsdl2-mixer-dev:i386 libsdl2-image-dev:i386 libpng12-dev:i386 || return 1
+    sudo apt-get purge -fy sbt || return 1
+    sudo apt-get -o Dpkg::Options::="--force-overwrite" -fy --no-install-recommends install gcc:i386 g++:i386 cpp:i386 g++-4.8:i386 gcc-4.8:i386 cpp-4.8:i386 binutils:i386 zlib1g-dev:i386 libsdl2-dev:i386 libsdl2-mixer-dev:i386 libsdl2-image-dev:i386 libpng12-dev:i386 || return 1
     make PLATFORM=linux32 PLATFORM_BIN=x86 INSTDIR=${SEMABUILD_DIR}/linux/bin/x86 CFLAGS=-m32 CXXFLAGS=-m32 LDFLAGS=-m32 -C src clean install || return 1
     return 0
 }
@@ -54,13 +64,12 @@ semabuild_integrate() {
         else
             SEMABUILD_MODDIR="${SEMABUILD_PWD}/data/${i}"
             echo "module ${i} updating.."
-            git submodule init "data/${i}"
-            git submodule update "data/${i}"
+            git submodule update --init --depth 1 "data/${i}" || return 1
         fi
         pushd "${SEMABUILD_MODDIR}" || return 1
         echo "module ${i} processing.."
         SEMABUILD_HASH=`git rev-parse HEAD` || return 1
-        SEMABUILD_LAST=`curl --fail --silent "${SEMABUILD_SOURCE}/${BRANCH_NAME}/${i}.txt"`
+        SEMABUILD_LAST=`curl --connect-timeout 30 -L -k -f "${SEMABUILD_SOURCE}/${BRANCH_NAME}/${i}.txt"`
         echo "module ${i} compare: ${SEMABUILD_LAST} -> ${SEMABUILD_HASH}"
         if [ -n "${SEMABUILD_HASH}" ] && [ "${SEMABUILD_HASH}" != "${SEMABUILD_LAST}" ]; then
             echo "module ${i} updated, syncing.."
@@ -68,7 +77,7 @@ semabuild_integrate() {
             SEMABUILD_DEPLOY="true"
             if [ "${i}" = "base" ]; then
                 echo "module ${i} checking for source modifications.."
-                SEMABUILD_BINS=`curl --fail --silent "${SEMABUILD_SOURCE}/${BRANCH_NAME}/bins.txt"` || return 1
+                SEMABUILD_BINS=`curl --connect-timeout 30 -L -k -f "${SEMABUILD_SOURCE}/${BRANCH_NAME}/bins.txt"` || return 1
                 SEMABUILD_CHANGES=`git diff --name-only HEAD ${SEMABUILD_BINS} -- src | egrep '\.h$|\.c$|\.cpp$|Makefile$'`
                 if [ -n "${SEMABUILD_CHANGES}" ]; then
                     echo "module ${i} has modified source files:"
@@ -94,11 +103,38 @@ semabuild_process() {
     return 0
 }
 
+semabuild_appimage() {
+    git clone --depth 1 "${SEMABUILD_APPIMAGE}" appimage || return 1
+    pushd appimage || return 1
+    export BRANCH="${BRANCH_NAME}"
+    export ARCH=x86_64
+    export COMMIT=${REVISION}
+    export BUILD_SERVER=1
+    export BUILD_CLIENT=1
+    export PLATFORM_BUILD=${SEMAPHORE_BUILD_NUMBER}
+    export PLATFORM_BRANCH="${BRANCH_NAME}"
+    export PLATFORM_REVISION="${REVISION}"
+    export NO_UPDATE=true
+    export BUILD="${SEMABUILD_PWD}"
+    bash build-appimages.sh || return 1
+    export GITHUB_TOKEN="${GITHUB_TOKEN}"
+    export REPO_SLUG="${SEMABUILD_APPIMAGE_GH_DEST}"
+    export COMMIT=$(git rev-parse ${REVISION})
+    bash github-release.sh || return 1
+    popd || return 1
+    return 0
+}
+
 semabuild_deploy() {
     echo "deploying ${BRANCH_NAME}..."
     echo "${SEMABUILD_ALLMODS}" > "${SEMABUILD_DIR}/mods.txt"
     pushd "${SEMABUILD_BUILD}" || return 1
-    ${SEMABUILD_SCP} -r "${BRANCH_NAME}" "${SEMABUILD_TARGET}" || return 1
+    git stash || return 1
+    git pull || return 1
+    git stash apply || return 1
+    git commit -a -m "Build ${BRANCH_NAME}:${SEMAPHORE_BUILD_NUMBER}" || return 1
+    git push -u origin master || return 1
+    popd || return 1
     return 0
 }
 
@@ -106,5 +142,12 @@ semabuild_setup || exit 1
 semabuild_process || exit 1
 if [ "${SEMABUILD_DEPLOY}" = "true" ]; then
     semabuild_deploy || exit 1
+    if [ "${BRANCH_NAME}" = master ] || [ "${BRANCH_NAME}" = stable ]; then
+        echo "building ${BRANCH_NAME} appimages..."
+        sudo ${SEMABUILD_APT} -fy install build-essential multiarch-support gcc-multilib g++-multilib zlib1g-dev libsdl2-dev libsdl2-mixer-dev libsdl2-image-dev jq zsync || exit 1
+        pushd "${HOME}" || return 1
+        semabuild_appimage || exit 1
+        popd || return 1
+    fi
 fi
 echo "done."
