@@ -424,13 +424,13 @@ extern void warpragdoll(dynent *d, const vec &vel, const vec &offset = vec(0, 0,
 extern void twitchragdoll(dynent *d, float vel);
 
 // server
-#define MAXCLIENTS 256                  // in a multiplayer game, can be arbitrarily changed
-#define MAXTRANS 5000                  // max amount of data to swallow in 1 go
+#define MAXCLIENTS 256  // in a multiplayer game, can be arbitrarily changed
+#define MAXTRANS 5000   // max amount of data to swallow in 1 go
+#define MAXSDESCLEN 80  // max length of server description field
 
-enum { DISC_NONE = 0, DISC_EOP, DISC_CN, DISC_KICK, DISC_MSGERR, DISC_IPBAN, DISC_PRIVATE, DISC_PASSWORD, DISC_PURE, DISC_MAXCLIENTS, DISC_INCOMPATIBLE, DISC_TIMEOUT, DISC_OVERFLOW, DISC_SHUTDOWN, DISC_HOSTFAIL, DISC_NUM };
+enum { DISC_NONE = 0, DISC_EOP, DISC_CN, DISC_KICK, DISC_MSGERR, DISC_IPBAN, DISC_PRIVATE, DISC_PASSWORD, DISC_PURE, DISC_MAXCLIENTS, DISC_INCOMPATIBLE, DISC_TIMEOUT, DISC_OVERFLOW, DISC_SHUTDOWN, DISC_HOSTFAIL, DISC_AUTH, DISC_NUM };
 
 extern void *getinfo(int i);
-extern const char *gethostname(int i);
 extern const char *gethostip(int i);
 extern void sendf(int cn, int chan, const char *format, ...);
 extern void sendfile(int cn, int chan, stream *file, const char *format = "", ...);
@@ -468,7 +468,6 @@ struct serverinfo
     enum
     {
         MAXPINGS = 3,
-
         WAITING = INT_MAX
     };
     enum { UNRESOLVED = 0, RESOLVING, RESOLVED };
@@ -652,7 +651,7 @@ struct clientdata
     int type;
     int num;
     ENetPeer *peer;
-    string hostname, hostip;
+    string hostip;
     void *info;
 };
 
