@@ -20,28 +20,11 @@ void operator delete(void *p) { if(p) free(p); }
 
 void operator delete[](void *p) { if(p) free(p); }
 
-#ifndef WIN32
-#include <unistd.h>
-#endif
-
-int guessnumcpus()
-{
-    int numcpus = 1;
-#ifdef WIN32
-    SYSTEM_INFO info;
-    GetSystemInfo(&info);
-    numcpus = (int)info.dwNumberOfProcessors;
-#elif defined(_SC_NPROCESSORS_ONLN)
-    numcpus = (int)sysconf(_SC_NPROCESSORS_ONLN);
-#endif
-    return max(numcpus, 1);
-}
-    
 ////////////////////////// rnd numbers ////////////////////////////////////////
 
-#define N (624)             
-#define M (397)                
-#define K (0x9908B0DFU)       
+#define N (624)
+#define M (397)
+#define K (0x9908B0DFU)
 
 static uint state[N];
 static int next = N;
@@ -173,4 +156,3 @@ void getstring(char *text, ucharbuf &p, size_t len)
     }
     while(*t++);
 }
-
