@@ -11,6 +11,7 @@
 typedef unsigned char uchar;
 typedef unsigned short ushort;
 typedef unsigned int uint;
+typedef unsigned long ulong;
 typedef signed long long int llong;
 typedef unsigned long long int ullong;
 
@@ -63,12 +64,36 @@ static inline T min(T a, T b)
 {
     return a < b ? a : b;
 }
-template<class T>
-static inline T clamp(T a, T b, T c)
+template<class T, class U>
+static inline T clamp(T a, U b, U c)
 {
-    return max(b, min(a, c));
+    return max(T(b), min(a, T(c)));
 }
 
+#ifdef __GNUC__
+#define bitscan(mask) (__builtin_ffs(mask)-1)
+#else
+#ifdef WIN32
+#pragma intrinsic(_BitScanForward)
+static inline int bitscan(uint mask)
+{
+    ulong i;
+    return _BitScanForward(&i, mask) ? i : -1;
+}
+#else
+static inline int bitscan(uint mask)
+{
+    if(!mask) return -1;
+    int i = 1;
+    if(!(mask&0xFFFF)) { i += 16; mask >>= 16; }
+    if(!(mask&0xFF)) { i += 8; mask >>= 8; }
+    if(!(mask&0xF)) { i += 4; mask >>= 4; }
+    if(!(mask&3)) { i += 2; mask >>= 2; }
+    return i - (mask&1);
+}
+#endif
+#endif
+
 #define rnd(x) ((int)(randomMT()&0x7FFFFFFF)%(x))
 #define rndscale(x) (float((randomMT()&0x7FFFFFFF)*double(x)/double(0x7FFFFFFF)))
 #define detrnd(s, x) ((int)(((((uint)(s))*1103515245+12345)>>16)%(x)))
@@ -136,30 +161,22 @@ typedef char string[MAXSTRLEN];
 #define BIGSTRLEN 4096
 typedef char bigstring[BIGSTRLEN];
 
-inline void vformatstring(char *d, const char *fmt, va_list v, int len = MAXSTRLEN)
-{
-    _vsnprintf(d, len, fmt, v);
-    d[len-1] = 0;
-}
-inline void vformatbigstring(char *d, const char *fmt, va_list v, int len = BIGSTRLEN) { vformatstring(d, fmt, v, len); }
+inline void vformatstring(char *d, const char *fmt, va_list v, int len) { _vsnprintf(d, len, fmt, v); d[len-1] = 0; }
+template<size_t N> inline void vformatstring(char (&d)[N], const char *fmt, va_list v) { vformatstring(d, fmt, v, N); }
 
-inline char *copystring(char *d, const char *s, size_t len = MAXSTRLEN)
+inline char *copystring(char *d, const char *s, size_t len)
 {
-    size_t slen = min(strlen(s)+1, len);
+    size_t slen = min(strlen(s), len-1);
     memcpy(d, s, slen);
-    d[slen-1] = 0;
+    d[slen] = 0;
     return d;
 }
-inline char *copybigstring(char *d, const char *s, size_t len = BIGSTRLEN) { return copystring(d, s, len); }
+template<size_t N> inline char *copystring(char (&d)[N], const char *s) { return copystring(d, s, N); }
 
-inline char *concatstring(char *d, const char *s, size_t len = MAXSTRLEN)
-{
-    size_t used = strlen(d);
-    return used < len ? copystring(d+used, s, len-used) : d;
-}
-inline char *concatbigstring(char *d, const char *s, size_t len = BIGSTRLEN) { return concatstring(d, s, len); }
+inline char *concatstring(char *d, const char *s, size_t len) { size_t used = strlen(d); return used < len ? copystring(d+used, s, len-used) : d; }
+template<size_t N> inline char *concatstring(char (&d)[N], const char *s) { return concatstring(d, s, N); }
 
-inline char *prependstring(char *d, const char *s, size_t len = MAXSTRLEN)
+inline char *prependstring(char *d, const char *s, size_t len)
 {
     size_t slen = min(strlen(s), len);
     memmove(&d[slen], d, min(len - slen, strlen(d) + 1));
@@ -167,55 +184,41 @@ inline char *prependstring(char *d, const char *s, size_t len = MAXSTRLEN)
     d[len-1] = 0;
     return d;
 }
-inline char *prependbigstring(char *d, const char *s, size_t len = BIGSTRLEN) { return prependstring(d, s, len); }
+template<size_t N> inline char *prependstring(char (&d)[N], const char *s) { return prependstring(d, s, N); }
 
-struct stringformatter
+inline void nformatstring(char *d, int len, const char *fmt, ...) PRINTFARGS(3, 4);
+inline void nformatstring(char *d, int len, const char *fmt, ...)
 {
-    char *buf;
-    stringformatter(char *buf): buf((char *)buf) {}
-    void operator()(const char *fmt, ...) PRINTFARGS(2, 3)
-    {
-        va_list v;
-        va_start(v, fmt);
-        vformatstring(buf, fmt, v);
-        va_end(v);
-    }
-    void operator()(int len, const char *fmt, ...) PRINTFARGS(3, 4)
-    {
-        va_list v;
-        va_start(v, fmt);
-        vformatstring(buf, fmt, v, len+1);
-        va_end(v);
-    }
-};
+    va_list v;
+    va_start(v, fmt);
+    vformatstring(d, fmt, v, len);
+    va_end(v);
+}
 
-#define formatstring(d) stringformatter((char *)d)
-#define defformatstring(d) string d; formatstring(d)
+template<size_t N> inline void formatstring(char (&d)[N], const char *fmt, ...) PRINTFARGS(2, 3);
+template<size_t N> inline void formatstring(char (&d)[N], const char *fmt, ...)
+{
+    va_list v;
+    va_start(v, fmt);
+    vformatstring(d, fmt, v, int(N));
+    va_end(v);
+}
+
+template<size_t N> inline void concformatstring(char (&d)[N], const char *fmt, ...) PRINTFARGS(2, 3);
+template<size_t N> inline void concformatstring(char (&d)[N], const char *fmt, ...)
+{
+    va_list v;
+    va_start(v, fmt);
+    int len = strlen(d);
+    vformatstring(d + len, fmt, v, int(N) - len);
+    va_end(v);
+}
+
+#define defformatstring(d,...) string d; formatstring(d, __VA_ARGS__)
 #define defvformatstring(d,last,fmt) string d; { va_list ap; va_start(ap, last); vformatstring(d, fmt, ap); va_end(ap); }
 
-struct bigstringformatter
-{
-    char *buf;
-    bigstringformatter(char *buf): buf((char *)buf) {}
-    void operator()(const char *fmt, ...) PRINTFARGS(2, 3)
-    {
-        va_list v;
-        va_start(v, fmt);
-        vformatbigstring(buf, fmt, v);
-        va_end(v);
-    }
-    void operator()(int len, const char *fmt, ...) PRINTFARGS(3, 4)
-    {
-        va_list v;
-        va_start(v, fmt);
-        vformatbigstring(buf, fmt, v, len+1);
-        va_end(v);
-    }
-};
-
-#define formatbigstring(d) bigstringformatter((char *)d)
-#define defformatbigstring(d) bigstring d; formatbigstring(d)
-#define defvformatbigstring(d,last,fmt) bigstring d; { va_list ap; va_start(ap, last); vformatbigstring(d, fmt, ap); va_end(ap); }
+#define defformatbigstring(d,...) bigstring d; formatstring(d, __VA_ARGS__)
+#define defvformatbigstring(d,last,fmt) bigstring d; { va_list ap; va_start(ap, last); vformatstring(d, fmt, ap); va_end(ap); }
 
 template<size_t N> inline bool matchstring(const char *s, size_t len, const char (&d)[N])
 {
@@ -251,6 +254,19 @@ struct databuf
     template<class U>
     databuf(T *buf, U maxlen) : buf(buf), len(0), maxlen((int)maxlen), flags(0) {}
 
+    void reset()
+    {
+        len = 0;
+        flags = 0;
+    }
+
+    void reset(T *buf_, int maxlen_)
+    {
+        reset();
+        buf = buf_;
+        maxlen = maxlen_;
+    }
+
     const T &get()
     {
         static T overreadval = 0;
@@ -266,6 +282,13 @@ struct databuf
         return databuf(&buf[len-sz], sz);
     }
 
+    T *pad(int numvals)
+    {
+        T *vals = &buf[len];
+        len += min(numvals, maxlen-len);
+        return vals;
+    }
+
     void put(const T &val)
     {
         if(len<maxlen) buf[len++] = val;
@@ -296,12 +319,15 @@ struct databuf
         len = max(len-n, 0);
     }
 
+    T *getbuf() const { return buf; }
     bool empty() const { return len==0; }
     int length() const { return len; }
     int remaining() const { return maxlen-len; }
     bool overread() const { return (flags&OVERREAD)!=0; }
     bool overwrote() const { return (flags&OVERWROTE)!=0; }
 
+    bool check(int n) { return remaining() >= n; }
+
     void forceoverread()
     {
         len = maxlen;
@@ -373,8 +399,19 @@ struct packetbuf : ucharbuf
 template<class T>
 static inline float heapscore(const T &n) { return n; }
 
-template<class T>
-static inline bool compareless(const T &x, const T &y) { return x < y; }
+struct sortless
+{
+    template<class T> bool operator()(const T &x, const T &y) const { return x < y; }
+    bool operator()(char *x, char *y) const { return strcmp(x, y) < 0; }
+    bool operator()(const char *x, const char *y) const { return strcmp(x, y) < 0; }
+};
+
+struct sortnameless
+{
+    template<class T> bool operator()(const T &x, const T &y) const { return sortless()(x.name, y.name); }
+    template<class T> bool operator()(T *x, T *y) const { return sortless()(x->name, y->name); }
+    template<class T> bool operator()(const T *x, const T *y) const { return sortless()(x->name, y->name); }
+};
 
 template<class T, class F>
 static inline void insertionsort(T *start, T *end, F fun)
@@ -403,7 +440,7 @@ static inline void insertionsort(T *buf, int n, F fun)
 template<class T>
 static inline void insertionsort(T *buf, int n)
 {
-    insertionsort(buf, buf+n, compareless<T>);
+    insertionsort(buf, buf+n, sortless());
 }
 
 template<class T, class F>
@@ -457,7 +494,7 @@ static inline void quicksort(T *buf, int n, F fun)
 template<class T>
 static inline void quicksort(T *buf, int n)
 {
-    quicksort(buf, buf+n, compareless<T>);
+    quicksort(buf, buf+n, sortless());
 }
 
 template<class T> struct isclass
@@ -479,6 +516,47 @@ static inline bool htcmp(const char *x, const char *y)
     return !strcmp(x, y);
 }
 
+struct stringslice
+{
+    const char *str;
+    int len;
+    stringslice() {}
+    stringslice(const char *str, int len) : str(str), len(len) {}
+    stringslice(const char *str, const char *end) : str(str), len(int(end-str)) {}
+
+    const char *end() const { return &str[len]; }
+};
+
+inline char *newstring(const stringslice &s) { return newstring(s.str, s.len); }
+inline const char *stringptr(const char *s) { return s; }
+inline const char *stringptr(const stringslice &s) { return s.str; }
+inline int stringlen(const char *s) { return int(strlen(s)); }
+inline int stringlen(const stringslice &s) { return s.len; }
+
+inline char *copystring(char *d, const stringslice &s, size_t len)
+{
+    size_t slen = min(size_t(s.len), len-1);
+    memcpy(d, s.str, slen);
+    d[slen] = 0;
+    return d;
+}
+template<size_t N> inline char *copystring(char (&d)[N], const stringslice &s) { return copystring(d, s, N); }
+
+static inline uint memhash(const void *ptr, int len)
+{
+    const uchar *data = (const uchar *)ptr;
+    uint h = 5381;
+    loopi(len) h = ((h<<5)+h)^data[i];
+    return h;
+}
+
+static inline uint hthash(const stringslice &s) { return memhash(s.str, s.len); }
+
+static inline bool htcmp(const stringslice &x, const char *y)
+{
+    return x.len == (int)strlen(y) && !memcmp(x.str, y, x.len);
+}
+
 static inline uint hthash(int key)
 {
     return key;
@@ -577,6 +655,7 @@ template <class T> struct vector
 
     T &pop() { return buf[--ulen]; }
     T &last() { return buf[ulen-1]; }
+    T &first() { return buf[0]; }
     void drop() { ulen--; buf[ulen].~T(); }
     bool empty() const { return ulen==0; }
 
@@ -603,7 +682,8 @@ template <class T> struct vector
         quicksort(&buf[i], n < 0 ? ulen-i : n, fun);
     }
 
-    void sort() { sort(compareless<T>); }
+    void sort() { sort(sortless()); }
+    void sortname() { sort(sortnameless()); }
 
     void growbuf(int sz)
     {
@@ -622,7 +702,7 @@ template <class T> struct vector
 
     databuf<T> reserve(int sz)
     {
-        if(ulen+sz > alen) growbuf(ulen+sz);
+        if(alen-ulen < sz) growbuf(ulen+sz);
         return databuf<T>(&buf[ulen], sz);
     }
 
@@ -718,7 +798,7 @@ template <class T> struct vector
 
     T *insert(int i, const T *e, int n)
     {
-        if(ulen+n>alen) growbuf(ulen+n);
+        if(alen-ulen < n) growbuf(ulen+n);
         loopj(n) add(T());
         for(int p = ulen-1; p>=i+n; p--) buf[p] = buf[p-n];
         loopj(n) buf[i+j] = e[j];
@@ -915,6 +995,7 @@ template <class T> struct smallvector
     bool inrange(int i) const { return i>=0 && i<len; }
 
     T &last() { return buf[len-1]; }
+    T &first() { return buf[0]; }
     bool empty() const { return len==0; }
     int length() const { return len; }
     T &operator[](int i) { ASSERT(i>=0 && i<len); return buf[i]; }
@@ -938,14 +1019,15 @@ template <class T> struct smallvector
     }
 };
 
-template<class T> struct hashset
+template<class H, class E, class K, class T> struct hashbase
 {
-    typedef T elem;
-    typedef const T const_elem;
+    typedef E elemtype;
+    typedef K keytype;
+    typedef T datatype;
 
     enum { CHUNKSIZE = 64 };
 
-    struct chain { T elem; chain *next; };
+    struct chain { E elem; chain *next; };
     struct chainchunk { chain chains[CHUNKSIZE]; chainchunk *next; };
 
     int size;
@@ -955,17 +1037,19 @@ template<class T> struct hashset
     chainchunk *chunks;
     chain *unused;
 
-    hashset(int size = 1<<10)
+    enum { DEFAULTSIZE = 1<<10 };
+
+    hashbase(int size = DEFAULTSIZE)
       : size(size)
     {
         numelems = 0;
         chunks = NULL;
         unused = NULL;
         chains = new chain *[size];
-        loopi(size) chains[i] = NULL;
+        memset(chains, 0, size*sizeof(chain *));
     }
 
-    ~hashset()
+    ~hashbase()
     {
         DELETEA(chains);
         deletechunks();
@@ -990,55 +1074,63 @@ template<class T> struct hashset
         return c;
     }
 
-    #define HTFIND(key, success, fail) \
+    template<class U>
+    T &insert(uint h, const U &key)
+    {
+        chain *c = insert(h);
+        H::setkey(c->elem, key);
+        return H::getdata(c->elem);
+    }
+
+    #define HTFIND(success, fail) \
         uint h = hthash(key)&(this->size-1); \
         for(chain *c = this->chains[h]; c; c = c->next) \
         { \
-            if(htcmp(key, c->elem)) return (success); \
+            if(htcmp(key, H::getkey(c->elem))) return success H::getdata(c->elem); \
         } \
         return (fail);
 
-    template<class K>
-    T *access(const K &key)
+    template<class U>
+    T *access(const U &key)
     {
-        HTFIND(key, &c->elem, NULL);
+        HTFIND(&, NULL);
     }
 
-    template<class K, class E>
-    T &access(const K &key, const E &elem)
+    template<class U, class V>
+    T &access(const U &key, const V &elem)
     {
-        HTFIND(key, c->elem, insert(h)->elem = elem);
+        HTFIND( , insert(h, key) = elem);
     }
 
-    template<class K>
-    T &operator[](const K &key)
+    template<class U>
+    T &operator[](const U &key)
     {
-        HTFIND(key, c->elem, insert(h)->elem);
+        HTFIND( , insert(h, key));
     }
 
-    template<class K>
-    T &find(const K &key, T &notfound)
+    template<class U>
+    T &find(const U &key, T &notfound)
     {
-        HTFIND(key, c->elem, notfound);
+        HTFIND( , notfound);
     }
 
-    template<class K>
-    const T &find(const K &key, const T &notfound)
+    template<class U>
+    const T &find(const U &key, const T &notfound)
     {
-        HTFIND(key, c->elem, notfound);
+        HTFIND( , notfound);
     }
 
-    template<class K>
-    bool remove(const K &key)
+    template<class U>
+    bool remove(const U &key)
     {
         uint h = hthash(key)&(size-1);
         for(chain **p = &chains[h], *c = chains[h]; c; p = &c->next, c = c->next)
         {
-            if(htcmp(key, c->elem))
+            if(htcmp(key, H::getkey(c->elem)))
             {
                 *p = c->next;
-                c->elem.~T();
-                new (&c->elem) T;
+                c->elem.~E();
+                new (&c->elem) E;
                 c->next = unused;
                 unused = c;
                 numelems--;
@@ -1060,86 +1152,72 @@ template<class T> struct hashset
     void clear()
     {
         if(!numelems) return;
-        loopi(size) chains[i] = NULL;
+        memset(chains, 0, size*sizeof(chain *));
         numelems = 0;
         unused = NULL;
         deletechunks();
     }
 
-    static inline chain *getnext(void *i) { return ((chain *)i)->next; }
-    static inline T &getdata(void *i) { return ((chain *)i)->elem; }
+    static inline chain *enumnext(void *i) { return ((chain *)i)->next; }
+    static inline K &enumkey(void *i) { return H::getkey(((chain *)i)->elem); }
+    static inline T &enumdata(void *i) { return H::getdata(((chain *)i)->elem); }
+};
+
+template<class T> struct hashset : hashbase<hashset<T>, T, T, T>
+{
+    typedef hashbase<hashset<T>, T, T, T> basetype;
+
+    hashset(int size = basetype::DEFAULTSIZE) : basetype(size) {}
+
+    static inline const T &getkey(const T &elem) { return elem; }
+    static inline T &getdata(T &elem) { return elem; }
+    template<class K> static inline void setkey(T &elem, const K &key) {}
+
+    template<class V>
+    T &add(const V &elem)
+    {
+        return basetype::access(elem, elem);
+    }
+};
+
+template<class T> struct hashnameset : hashbase<hashnameset<T>, T, const char *, T>
+{
+    typedef hashbase<hashnameset<T>, T, const char *, T> basetype;
+
+    hashnameset(int size = basetype::DEFAULTSIZE) : basetype(size) {}
+
+    template<class U> static inline const char *getkey(const U &elem) { return elem.name; }
+    template<class U> static inline const char *getkey(U *elem) { return elem->name; }
+    static inline T &getdata(T &elem) { return elem; }
+    template<class K> static inline void setkey(T &elem, const K &key) {}
+
+    template<class V>
+    T &add(const V &elem)
+    {
+        return basetype::access(getkey(elem), elem);
+    }
 };
 
 template<class K, class T> struct hashtableentry
 {
     K key;
     T data;
-
-    hashtableentry() {}
-    hashtableentry(const K &key, const T &data) : key(key), data(data) {}
 };
 
-template<class U, class K, class T>
-static inline bool htcmp(const U *x, const hashtableentry<K, T> &y)
+template<class K, class T> struct hashtable : hashbase<hashtable<K, T>, hashtableentry<K, T>, K, T>
 {
-    return htcmp(x, y.key);
-}
+    typedef hashbase<hashtable<K, T>, hashtableentry<K, T>, K, T> basetype;
+    typedef typename basetype::elemtype elemtype;
 
-template<class U, class K, class T>
-static inline bool htcmp(const U &x, const hashtableentry<K, T> &y)
-{
-    return htcmp(x, y.key);
-}
+    hashtable(int size = basetype::DEFAULTSIZE) : basetype(size) {}
 
-template<class K, class T> struct hashtable : hashset<hashtableentry<K, T> >
-{
-    typedef hashtableentry<K, T> entry;
-    typedef struct hashset<entry>::chain chain;
-    typedef K key;
-    typedef T value;
-
-    hashtable(int size = 1<<10) : hashset<entry>(size) {}
-
-    entry &insert(const K &key, uint h)
-    {
-        chain *c = hashset<entry>::insert(h);
-        c->elem.key = key;
-        return c->elem;
-    }
-
-    T *access(const K &key)
-    {
-        HTFIND(key, &c->elem.data, NULL);
-    }
-
-    T &access(const K &key, const T &data)
-    {
-        HTFIND(key, c->elem.data, insert(key, h).data = data);
-    }
-
-    T &operator[](const K &key)
-    {
-        HTFIND(key, c->elem.data, insert(key, h).data);
-    }
-
-    T &find(const K &key, T &notfound)
-    {
-        HTFIND(key, c->elem.data, notfound);
-    }
-
-    const T &find(const K &key, const T &notfound)
-    {
-        HTFIND(key, c->elem.data, notfound);
-    }
-
-    static inline chain *getnext(void *i) { return ((chain *)i)->next; }
-    static inline K &getkey(void *i) { return ((chain *)i)->elem.key; }
-    static inline T &getdata(void *i) { return ((chain *)i)->elem.data; }
+    static inline K &getkey(elemtype &elem) { return elem.key; }
+    static inline T &getdata(elemtype &elem) { return elem.data; }
+    template<class U> static inline void setkey(elemtype &elem, const U &key) { elem.key = key; }
 };
 
-#define enumerates(ht,t,e,b)      loopi((ht).size)  for(hashset<t>::chain *enumc = (ht).chains[i]; enumc;) { t &e = enumc->elem; enumc = enumc->next; b; }
-#define enumeratekt(ht,k,e,t,f,b) loopi((ht).size)  for(hashtable<k,t>::chain *enumc = (ht).chains[i]; enumc;) { const hashtable<k,t>::key &e = enumc->elem.key; t &f = enumc->elem.data; enumc = enumc->next; b; }
-#define enumerate(ht,t,e,b)       loopi((ht).size) for(void *enumc = (ht).chains[i]; enumc;) { t &e = (ht).getdata(enumc); enumc = (ht).getnext(enumc); b; }
+#define enumeratekt(ht,k,e,t,f,b) loopi((ht).size) for(void *ec = (ht).chains[i]; ec;) { k &e = (ht).enumkey(ec); t &f = (ht).enumdata(ec); ec = (ht).enumnext(ec); b; }
+#define enumerate(ht,t,e,b)       loopi((ht).size) for(void *ec = (ht).chains[i]; ec;) { t &e = (ht).enumdata(ec); ec = (ht).enumnext(ec); b; }
 
 struct unionfind
 {
@@ -1297,6 +1375,14 @@ template<class T> inline void endiansame(T *buf, size_t len) {}
 #define lilswap endianswap
 #define bigswap endiansame
 #endif
+#elif defined(__BYTE_ORDER__)
+#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
+#define lilswap endiansame
+#define bigswap endianswap
+#else
+#define lilswap endianswap
+#define bigswap endiansame
+#endif
 #else
 template<class T> inline T lilswap(T n) { return *(const uchar *)&islittleendian ? n : endianswap(n); }
 template<class T> inline void lilswap(T *buf, size_t len) { if(!*(const uchar *)&islittleendian) endianswap(buf, len); }
@@ -1389,6 +1475,7 @@ static inline int iscubealpha(uchar c) { return cubectype[c]&CT_ALPHA; }
 static inline int iscubealnum(uchar c) { return cubectype[c]&(CT_ALPHA|CT_DIGIT); }
 static inline int iscubelower(uchar c) { return cubectype[c]&CT_LOWER; }
 static inline int iscubeupper(uchar c) { return cubectype[c]&CT_UPPER; }
+static inline int iscubepunct(uchar c) { return cubectype[c] == CT_PRINT; }
 static inline int cube2uni(uchar c)
 {
     extern const int cube2unichars[256];
@@ -1413,6 +1500,8 @@ static inline uchar cubeupper(uchar c)
 extern size_t decodeutf8(uchar *dst, size_t dstlen, const uchar *src, size_t srclen, size_t *carry = NULL);
 extern size_t encodeutf8(uchar *dstbuf, size_t dstlen, const uchar *srcbuf, size_t srclen, size_t *carry = NULL);
 
+extern int crcstream(stream *f);
+extern int crcfile(const char *s);
 extern char *makerelpath(const char *dir, const char *file, const char *prefix = NULL, const char *cmd = NULL);
 extern char *makefile(const char *s, const char *e = "", int revision = 0, int start = 1, bool store = false, bool skip = false);
 extern char *path(char *s, bool simple = false);
@@ -1421,6 +1510,7 @@ extern const char *parentdir(const char *directory);
 extern bool fileexists(const char *path, const char *mode);
 extern bool createdir(const char *path);
 extern void sethomedir(const char *dir);
+extern const char *gethomedir();
 extern void appendhomedir(const char *dir);
 extern void addpackagedir(const char *dir, int flags = 0);
 extern int maskpackagedirs(int flags);
@@ -1432,6 +1522,7 @@ extern stream *openfile(const char *filename, const char *mode);
 extern stream *opentempfile(const char *filename, const char *mode);
 extern stream *opengzfile(const char *filename, const char *mode, stream *file = NULL, int level = Z_BEST_COMPRESSION);
 extern stream *openutf8file(const char *filename, const char *mode, stream *file = NULL);
+extern char *loadstream(stream *f, size_t *size, bool utf8 = true);
 extern char *loadfile(const char *fn, size_t *size, bool utf8 = true);
 extern bool listdir(const char *dir, bool rel, const char *ext, vector<char *> &files);
 extern int listfiles(const char *dir, const char *ext, vector<char *> &files);
@@ -1441,7 +1532,6 @@ extern void backup(const char *fname, const char *ext, int revision = 0, int sta
 extern void endianswap(void *, int, int);
 extern void seedMT(uint seed);
 extern uint randomMT();
-extern int guessnumcpus();
 extern void putint(ucharbuf &p, int n);
 extern void putint(packetbuf &p, int n);
 extern void putint(vector<uchar> &p, int n);
@@ -1461,4 +1551,3 @@ extern void getstring(char *t, ucharbuf &p, size_t len);
 template<size_t N> static inline void getstring(char (&t)[N], ucharbuf &p) { getstring(t, p, N); }
 
 #endif
-
